#include "analysisFunction.h"
#include "shared.h"
#include "faidx.h"
#include "general.h"

class getFasta : public general{
private:
  char *ancName;
  char *refName;
  void getRef();
  void getAnc();
  faidx_t *faiRef;
  faidx_t *faiAnc;
public:
  char *getRef2(const loci *locs,int lens,int doCleanup);
  char *getRef1(const loci *locs,int lens,int doCleanup);
  getFasta(argStruct *arguments);
  ~getFasta();
  void printArg(FILE *argFile);
  void getOptions(argStruct *arguments);
  void run(funkyPars  *pars);//plugs in reference and ancestral
  void print(funkyPars *pars);
  void clean(funkyPars *pars);

};
void getFasta::printArg(FILE *argFile){
  fprintf(argFile,"%s:\n\n",__FILE__);
  fprintf(argFile,"-ref\t%s\n",refName);
  fprintf(argFile,"-anc\t%s\n",ancName);
  fprintf(argFile,"\n");

}

getFasta::~getFasta(){
  getRef1(NULL,0,1);//<-last indicate a cleanup
  getRef2(NULL,0,1);//<-last indicate a cleanup

  if(refName!=NULL&&faiRef!=NULL)
    fai_destroy(faiRef);
  if(ancName!=NULL&&faiAnc!=NULL)
    fai_destroy(faiAnc);

}


void getFasta::getOptions(argStruct *arguments){
  //default
  refName = NULL;
  ancName = NULL;
  //from command line
  ancName = angsd::getArg("-anc",ancName,arguments);
  refName = angsd::getArg("-ref",refName,arguments);
  //  fprintf(stderr,"refName=%s\tancName=%s\n",refName,ancName);


  if(ancName!=NULL)
    if(strcmp(ancName,"-999")==0){
      ancName=NULL;
      printArg(stdout);
      exit(0);
 
    }
  if(refName!=NULL)
    if(strcmp(refName,"-999")==0){
      refName=NULL;
      printArg(stdout);
      exit(0);
 
    }
  if(ancName==NULL|refName==NULL)
    return;
  printArg(arguments->argumentFile);

}

 getFasta::getFasta(argStruct *arguments){

   //   fprintf(stderr,"index=%d tot_index=%d\n",index,tot_index);
   getOptions(arguments);

  if(refName!=NULL)
    faiRef = fai_load(refName);
  if(ancName!=NULL)
    faiAnc = fai_load(ancName);
}

void getFasta::run(funkyPars *pars){
  if(refName!=NULL)
    pars->ref = getRef1(pars->sites,pars->numSites,0);
  if(ancName!=NULL)
    pars->anc = getRef2(pars->sites,pars->numSites,0);
  
}


char *getFasta::getRef1(const loci *locs,int lens,int doCleanup){
  static char *chrLast =NULL;
  static char *ref = NULL;
  static int ref_len = -1;
  if(doCleanup){
    free(chrLast);
    free(ref);
    return NULL;
  }


  //  fprintf(stderr,"\t-> Reading fasta ref/anc [%s] fname=%s\n",__FUNCTION__,fasta);

  char *ret=new char[lens];

  
  
  if(chrLast==NULL||(strcmp(chrLast,locs[0].chromo)!=0)){
    chrLast =strdup(locs[0].chromo);//THORFINN leak.. should fix.
    ref = faidx_fetch_seq(faiRef, chrLast, 0, 0x7fffffff, &ref_len);
    if(ref==NULL){
      fprintf(stderr,"Problems loading chr:%s from fasta\n",locs[0].chromo);
      //      exit(0);
    }
  }

  for(int i=0;i<lens;i++){
    if(strcmp(locs[i].chromo,chrLast)!=0){
      free(ref);
      chrLast=locs[i].chromo;
      ref=NULL;
      ref = faidx_fetch_seq(faiRef, chrLast, 0, 0x7fffffff, &ref_len);
      if(ref==NULL){
	fprintf(stderr,"[%s] Error loading reference alleles\n",__FUNCTION__);
	//exit(0);
      }
    }
    //ret[i] = refToInt[ref[locs[i].position-1]];//offseting by one, to make in work like samtools
    if(ref!=NULL)
      ret[i] = refToInt[ref[locs[i].position]];//offseting by one, to make in work like samtools
    else
      ret[i] = 4;
  }

  return ret;
}


char *getFasta::getRef2(const loci *locs,int lens,int doCleanup){
  static char *chrLast =NULL;
  static char *ref = NULL;
  static int ref_len = -1;
  if(doCleanup){
    free(chrLast);
    free(ref);
    return NULL;
  }


  //  fprintf(stderr,"\t-> Reading fasta ref/anc [%s] fname=%s\n",__FUNCTION__,fasta);

  char *ret=new char[lens];

 
  if(chrLast==NULL||(strcmp(chrLast,locs[0].chromo)!=0)){
    chrLast =strdup(locs[0].chromo);//THORFINN leak.. should fix.
    ref = faidx_fetch_seq(faiAnc, chrLast, 0, 0x7fffffff, &ref_len);
  }
  for(int i=0;i<lens;i++){
    if(strcmp(locs[i].chromo,chrLast)!=0){
      chrLast=locs[i].chromo;
      free(ref);
      ref=NULL;
      ref = faidx_fetch_seq(faiAnc, chrLast, 0, 0x7fffffff, &ref_len);
      if(ref==NULL){
	fprintf(stderr,"[%s] Error loading reference alleles, will use N/n for entire chromosome\n",__FUNCTION__);
	//	exit(0);
      }
    }
    if(ref!=NULL)
      ret[i] = refToInt[ref[locs[i].position]];//offseting by one, to make in work like samtools
    else
      ret[i] = 4;

  }

  return ret;
}

void getFasta::print(funkyPars *f){


}
void getFasta::clean(funkyPars *f){


}
