#include <vector>
#include <cstring>
#include <cstdlib>
#include <ctype.h>
#include <pthread.h>
#include "bams.h"
#include "kstring.h"

#include "utility.h"
#include "printer.h"
#include "indexer.h"
#include "mUpPile.h"
#include "threadpool2.cpp"

extern int SIG_COND;
static const char *bam_nt16_rev_table = "=ACMGRSVTWYHKDBN";

/*
  When merging different nodes, we are using a greedy but fast approach,
  however if we have a large genomic region with no data we are are allocating huge chunks.
  So the cutoff below will make a fallback to a slower but memsecure version
 */
#define BUG_THRES 1000000 //<- we allow 1mio sites to be allocated,

 

void cleanUpChunkyT(chunkyT *chk){
  //  fprintf(stderr,"[%s] nsties=%d\n\n",__FUNCTION__,chk->nSites);
  for(int s=0;s<chk->nSites;s++) {
    for(int i=0;i<chk->nSamples;i++) {
      if(chk->nd[s][i].l2!=0)
	//	fprintf(stderr,"l3\t%d\n",chk->nd[s][i].l2);
	for(int j=0;j<chk->nd[s][i].l2;j++){
	//	fprintf(stderr,"removing indels\n");
	dalloc_node(chk->nd[s][i].insert[j]);
      }
      free(chk->nd[s][i].insert);
      dalloc_node(chk->nd[s][i]);
    }
    delete [] chk->nd[s];
  }
  delete [] chk->nd;
  delete [] chk->refPos;
  delete chk;
  
}




node initNode(int l,int refPos,int arypos){
  //  fprintf(stderr,"arypos:%d\n",arypos);
  node nd;
  //  nd.isTail=nd.isHead=nd.refPos=-1;
  //  nd.len=0;
  nd.maxLen = l;
  nd.seq.s=NULL; nd.seq.l=nd.seq.m=0;
  nd.qs.s =NULL; nd.qs.l=nd.qs.m=0;
  nd.pos.s=NULL; nd.pos.l=nd.pos.m=0;
  nd.refPos = refPos;//not used anymore
  nd.depth = 0;
  return nd;
}


int actuallyReads =0;

tNode *allocNodeT(int l){
  tNode *d = new tNode;
  d->l = d->l2 = 0;
  d->m = l;
  fprintf(stderr,"d.m=%d\n",d->m);
  kroundup32(d->m);
  fprintf(stderr,"d.m=%d\n",d->m);
  d->seq=(char *)malloc(d->m);
  d->qs=(char *)malloc(d->m);
  d->posi=(char *)malloc(d->m);
  d->isop=(char *)malloc(d->m);
  d->mapQ=(char *)malloc(d->m);
  d->refPos=-999;
  d->insert = NULL;
  return d;
}

tNode initNodeT(int l,int refPos, int arypos){
  tNode d;
  d.l = d.l2 = d.m2 = 0;
  d.m = l;
  //  fprintf(stderr,"d.m=%d\n",d.m);
  kroundup32(d.m);
  //  fprintf(stderr,"d.m=%d\n",d.m);
  d.seq=(char *)malloc(d.m);
  d.qs=(char *)malloc(d.m);
  d.posi=(char *)malloc(d.m);
  d.isop=(char *)malloc(d.m);
  d.mapQ=(char *)malloc(d.m);
  d.refPos= -999;
  d.insert = NULL;
  return d;
}



bufReader initBufReader2(const char*fname){
  //  fprintf(stderr,"making bufreader:%s\n",fname);
  bufReader ret;
  ret.fname = strdup(fname);
  ret.fp = openBAM(ret.fname);
  pthread_mutex_init(&ret.bufMut,NULL);
  ret.hd = getHd(ret.fp);
  ret.isEOF =0;
  ret.nHasRead =0;
  ret.it.from_first=1;//iterator, used when supplying regions
  ret.it.finished=0;//iterator, used when supplying regions
  ret.it.off = NULL;
  ret.it.dasIndex = NULL;
  ret.itrPos =-1;
  return ret;
}
/*
  struct to represent stuff

  totalsize: 56bytes + m-bytes
  
  
*/


void realloc(char **c,int l,int m){
  char *tmp =(char *) malloc(m);
  memcpy(tmp,*c,l);
  free(*c);
  *c= tmp;
}

void realloc(tNode *d,int newsize){
  kroundup32(newsize);
  if(newsize<=d->m)
    fprintf(stderr,"[%s] problems newsize should be bigger than oldesize\n",__FUNCTION__);
  d->m=newsize;
  realloc(&(d->seq),d->l,d->m);
  realloc(&(d->qs),d->l,d->m);
  realloc(&(d->posi),d->l,d->m);
  realloc(&(d->isop),d->l,d->m);
  realloc(&(d->mapQ),d->l,d->m);
}

void dalloc_bufReader(bufReader &ret){
  free(ret.fname);
  free(ret.it.off);//cleanup offsets if used
  bgzf_close(ret.fp);
  dalloc(ret.hd);
}

/*
  compare all entries in the 2 headers, if difference return 0;
*/
int compHeader(aHead *hd1,aHead *hd2){
  if(0){
    if(hd1->l_text!=hd2->l_text)
      fprintf(stderr,"problem with l_text in header\n");
    if(memcmp(hd1->text,hd2->text,hd1->l_text)!=0)
      fprintf(stderr,"problem with text in header\n");
  }
  if(hd1->n_ref!=hd2->n_ref){
    fprintf(stderr,"problem with number of chromosomes in header\n");
    exit(0);
  }
  for(int i=0;i<hd1->n_ref;i++){
    if(strcmp(hd1->name[i],hd2->name[i])!=0){
      fprintf(stderr,"problem with chromosome ordering");
      exit(0);
    }
    if(hd1->l_ref[i]!=hd2->l_ref[i]){
      fprintf(stderr,"problem with chromosome ordering");
      exit(0);
    }
  }
  return 0;
}

bufReader *initializeBufReaders2(const std::vector<char *> vec){
  bufReader *ret = new bufReader[vec.size()];
  
  for(size_t i =0;i<vec.size();i++)
    ret[i] = initBufReader2(vec[i]);

  //now all readers are inialized, lets validate the header is the same
  for(size_t i=1;i<vec.size();i++)
    if(compHeader(ret[0].hd,ret[i].hd))
      printErr();

  //now all is fine return
  return ret;
}


void realloc(sglPool &ret,int l){
  //  fprintf(stderr,"[%s]reallocing pool from, ret.m=%d to ",__FUNCTION__,ret.m);
  ret.m =l;
  kroundup32(ret.m);
  //  fprintf(stderr," ret.m=%d\n",ret.m);
  if(0){
    ret.reads=(aRead *)realloc(ret.reads,sizeof(aRead)*ret.m);
    ret.first=(int *)realloc(ret.first,sizeof(int)*ret.m);
    ret.last=(int *)realloc(ret.last,sizeof(int)*ret.m);
  }else{
    aRead *tmp = new aRead[ret.m];
    memcpy(tmp,ret.reads,sizeof(aRead)*ret.l);
    delete [] ret.reads;
    ret.reads = tmp;
    
    int *tmpI = new int[ret.m];
    memcpy(tmpI,ret.first,sizeof(int)*ret.l);
    delete [] ret.first;
    ret.first = tmpI;

    
    tmpI = new int[ret.m];
    memcpy(tmpI,ret.last,sizeof(int)*ret.l);
    delete [] ret.last;
    ret.last = tmpI;
  }
}



void getPool7(BGZF *fp,int nReads,int &isEof,sglPool &ret,int refToRead,iter_t *it,int stop,int &rdObjEof,int &rdObjRegionDone) {
  if(ret.bufferedRead.refID!=-2 && ret.bufferedRead.pos>stop)
    return;
  if(ret.bufferedRead.refID!=-2 && ret.bufferedRead.refID!=refToRead){
    fprintf(stderr,"this happens once\n");
    rdObjRegionDone =1;
    // ret.regionDone = 1;
    return;
  }
    
  //  fprintf(stderr,"--------\n");
  if(ret.isEOF||rdObjEof){
    fprintf(stderr,"neverherer\n");
    fflush(stderr);
    exit(0);
    return;

  }
  //  ret.l=0;
  //  fprintf(stderr,"[%s] her\n",__FUNCTION__);
  extern int bam_iter_read1(BGZF *fp, iter_t *iter, aRead &b);

  if((nReads+ret.l)>ret.m)
    realloc(ret,nReads+ret.l);


  //int start =0;
  if(ret.bufferedRead.refID!=-2){//
    //start = 1;
    ret.reads[ret.l] = ret.bufferedRead;//copy constructor
    ret.first[ret.l] = ret.reads[ret.l].pos;
    ret.last[ret.l] = bam_calend(ret.reads[ret.l],getCig(&ret.reads[ret.l]));
    //    fprintf(stderr,"reading buffered\n");
    ret.bufferedRead.refID = -2;
    ret.l++;
    
  }


  //this is the awkward case this could cause an error in some very unlikely scenario.
  //whith the current buffer empty and the first read being a new chromosome. 
  //this is fixed good job monkey boy
  while(ret.l==0){
    //fprintf(stderr,"awkward\n");
    aRead &b = ret.reads[0];
    int tmp;
    b.vDat = new uint8_t[RLEN];
    if((tmp=bam_iter_read1(fp,it,b))<0){//FIXME
      if(tmp==-2){
	rdObjEof =1;
	//	ret.isEOF =1;
	isEof--;
      }else
	rdObjRegionDone =1;
      break;
    }
    if(b.nCig!=0){
      ret.first[ret.l] = b.pos;
      ret.last[ret.l] = bam_calend(b,getCig(&b));
      ret.l++;
      nReads--;
    }
  }

  // fprintf(stderr,"after awkward\n");
  /*
    now do the general loop,
    1) read an alignemnt calulate start and end pos
    check
    a) if same chromosome that a new read has a geq FINE
    b) if same but < then UNSORTED
    
  */ 
  int i=0;
  int tmp;
  int buffed=0;
  //  for( i=0;i<nReads;i++) {
  while(1) {
    //    fprintf(stderr,"ret.l+i=%d\tret.m-1=%d\n",i+ret.l,ret.m-1);
    if(i+ret.l>=(ret.m-1)){
      //      fprintf(stderr,"reallocing\n");
      ret.l += i;
      i=0;
      realloc(ret,ret.m+1);//will double buffer
    }

    aRead &b = ret.reads[i+ret.l];
    b.vDat = new uint8_t[RLEN];
    if((tmp=bam_iter_read1(fp,it,b))<0){
      if(tmp==-2){
	rdObjEof =1;
	isEof--;
      }else
	rdObjRegionDone =1;

      delete [] b.vDat;
      break;
    }if(b.nCig==0){//only get reads with CIGAR
      //      fprintf(stderr,"acigar\n");
      delete [] b.vDat;
      //      i--;
      continue;
    }
    //general fine case
    //    fprintf(stderr,"i=%d\trefToRead=%d\tb.refid=%d\tb.pos=%d\tret.first[last]=%d\tstop=%d ret.l=%d\n",i,refToRead,b.refID,b.pos,ret.first[ret.l+i-1],stop,ret.l);
    if((refToRead==b.refID)&&( b.pos >= ret.first[ret.l+i-1])&&(b.pos<stop)){
      ret.first[ret.l+i] = b.pos;
      ret.last[ret.l+i] = bam_calend(b,getCig(&b));
    }else if(b.pos>=stop){
      buffed=1;
      ret.bufferedRead = b;
      break;
    }
    else if(b.refID>refToRead){
      buffed=1;
      ret.bufferedRead = b;
      //      fprintf(stderr,"[%s] new chromosome detected will temporarliy stop reading at read # :%d\n",__FUNCTION__,i);
      //      ret.regionDone = 1;
      rdObjRegionDone =1;
      break;
    }else if(ret.first[ret.l+i-1]>b.pos){
      //      fprintf(stderr,"unsorted file detected will exit\n");
      fflush(stderr);
      exit(0);
    }
    i++;
  }
  ret.nReads =ret.l+i;
  ret.l = ret.nReads;
  //  fprintf(stderr,"after read sglfpool.l=%d\n",ret.l);
}



void getPool5(BGZF *fp,int nReads,int &isEof,sglPool &ret,int refToRead,iter_t *it) {
  //  fprintf(stderr,"\t->[%s] buffRefid=%d\trefToRead=%d\n",__FUNCTION__,ret.bufferedRead.refID,refToRead);
  if(ret.bufferedRead.refID!=-2 && ret.bufferedRead.refID!=refToRead){
    //    fprintf(stderr,"this happens once\n");
    ret.regionDone = 1;
    return;
  }
    
  //  fprintf(stderr,"--------\n");
  if(ret.isEOF)
    return;
  //  ret.l=0;
  extern int bam_iter_read1(BGZF *fp, iter_t *iter, aRead &b);

  if((nReads+ret.l)>ret.m)
    realloc(ret,nReads+ret.l);


  //int start =0;
  if(ret.bufferedRead.refID!=-2){//
    //start = 1;
    ret.reads[ret.l] = ret.bufferedRead;//copy constructor
    ret.first[ret.l] = ret.reads[ret.l].pos;
    ret.last[ret.l] = bam_calend(ret.reads[ret.l],getCig(&ret.reads[ret.l]));
    //    fprintf(stderr,"reading buffered\n");
    ret.bufferedRead.refID = -2;
    ret.l++;
    
  }
  //start by allocating the memeory we need

  for(int i=0;i<nReads;i++){
    ///    fprintf(stderr,"allocing:%d\n",ret.l+i);
    aRead b;
    b.vDat=new uint8_t[RLEN];
    ret.reads[ret.l+i] =b;
  }

  //this is the awkward case this could cause an error in some very unlikely scenario.
  //whith the current buffer empty and the first read being a new chromosome. 
  //this is fixed good job monkey boy
  while(ret.l==0){
    //fprintf(stderr,"awkward\n");
    aRead &b = ret.reads[0];
    int tmp;
    if((tmp=bam_iter_read1(fp,it,b))<0){//FIXME
      ret.isEOF =1;
      isEof--;
      break;
    }
    if(b.nCig!=0){
      ret.first[ret.l] = b.pos;
      ret.last[ret.l] = bam_calend(b,getCig(&b));
      ret.l++;
      nReads--;
    }
  }
  // fprintf(stderr,"after awkward\n");
  /*
    now do the general loop,
    1) read an alignemnt calulate start and end pos
    check
    a) if same chromosome that a new read has a geq FINE
    b) if same but < then UNSORTED
    
  */ 
  int i;
  int tmp;
  int buffed=0;
  for( i=0;i<nReads;i++){
    //fprintf(stderr,"i+ret.l=%d\n",i+ret.l);
    //    int block_size;
    aRead &b = ret.reads[i+ret.l];

    if((tmp=bam_iter_read1(fp,it,b))<0){
      ret.isEOF =1;
      isEof--;
      break;
    }if(b.nCig==0){//only get reads with CIGAR
      i--;
      continue;
    }
    //general fine case
    if((refToRead==b.refID)&&( b.pos >= ret.first[ret.l+i-1])){
      ret.first[ret.l+i] = b.pos;
      ret.last[ret.l+i] = bam_calend(b,getCig(&b));
    }else if(b.refID>refToRead){
      buffed=1;
      ret.bufferedRead = b;
      //      fprintf(stderr,"[%s] new chromosome detected will temporarliy stop reading at read # :%d\n",__FUNCTION__,i);
      ret.regionDone = 1;
      break;
    }else if(ret.first[ret.l+i-1]>b.pos){
      fprintf(stderr,"unsorted file detected will exit\n");
      exit(0);
    }
    
  }


  if(i!=nReads){
    for(int s=ret.l+i+buffed;s<ret.l+nReads;s++)
      delete [] ret.reads[s].vDat;
  }

  ret.nReads =ret.l+i;
  ret.l = ret.nReads;

}


void getPool555(BGZF *fp,int nReads,int &isEof,sglPool &ret,int refToRead,iter_t *it,int &rdObjEof,int &rdObjRegionDone) {

  fprintf(stderr,"\t->[%s] buffRefid=%d\trefToRead=%d\n",__FUNCTION__,ret.bufferedRead.refID,refToRead);
  if(ret.bufferedRead.refID!=-2 && ret.bufferedRead.refID!=refToRead){
    fprintf(stderr,"this happens once\n");
    rdObjRegionDone =1;
    return;
  }
    
  //  fprintf(stderr,"--------\n");
  if(rdObjEof)
    return;
  //  ret.l=0;
  extern int bam_iter_read1(BGZF *fp, iter_t *iter, aRead &b);

  if((nReads+ret.l)>ret.m){
    fprintf(stderr,"[%s:%s()] reallocing\n",__FILE__,__FUNCTION__);
    realloc(ret,nReads+ret.l);
  }

  //int start =0;
  if(ret.bufferedRead.refID!=-2){//
    //start = 1;
    ret.reads[ret.l] = ret.bufferedRead;//copy constructor
    ret.first[ret.l] = ret.reads[ret.l].pos;
    ret.last[ret.l] = bam_calend(ret.reads[ret.l],getCig(&ret.reads[ret.l]));
    //    fprintf(stderr,"reading buffered\n");
    ret.bufferedRead.refID = -2;
    ret.l++;
    
  }
  //start by allocating the memeory we need

  for(int i=0;i<nReads;i++){
    ///    fprintf(stderr,"allocing:%d\n",ret.l+i);
    aRead b;
    b.vDat=new uint8_t[RLEN];
    ret.reads[ret.l+i] =b;
  }

  //this is the awkward case this could cause an error in some very unlikely scenario.
  //whith the current buffer empty and the first read being a new chromosome. 
  //this is fixed good job monkey boy
  while(ret.l==0){
    //fprintf(stderr,"looping\n");
    //fprintf(stderr,"awkward\n");
    aRead &b = ret.reads[0];
    int tmp;
    if((tmp=bam_iter_read1(fp,it,b))<0){//FIXME
      if(tmp==-2){
	rdObjEof =1;
	isEof--;
      }else
	rdObjRegionDone =1;

      break;
    }
    if(b.nCig!=0){
      ret.first[ret.l] = b.pos;
      ret.last[ret.l] = bam_calend(b,getCig(&b));
      ret.l++;
      nReads--;
    }
  }
  //  fprintf(stderr,"after awkward: ret.l=%d\n",ret.l);
  /*
    now do the general loop,
    1) read an alignemnt calulate start and end pos
    check
    a) if same chromosome that a new read has a geq FINE
    b) if same but < then UNSORTED
    
  */ 
  int i;
  int tmp;
  int buffed=0;
  for( i=0;i<nReads;i++){
    //fprintf(stderr,"i+ret.l=%d\n",i+ret.l);
    //    int block_size;
    aRead &b = ret.reads[i+ret.l];

    if((tmp=bam_iter_read1(fp,it,b))<0){
      if(tmp==-2){
	rdObjEof =1;
	isEof--;
      }else
	rdObjRegionDone =1;
      break;
    }if(b.nCig==0){//only get reads with CIGAR
      i--;
      continue;
    }
    //general fine case
    if((refToRead==b.refID)&&( b.pos >= ret.first[ret.l+i-1])){
      ret.first[ret.l+i] = b.pos;
      ret.last[ret.l+i] = bam_calend(b,getCig(&b));
    }else if(b.refID>refToRead){
      buffed=1;
      ret.bufferedRead = b;
      //      fprintf(stderr,"[%s] new chromosome detected will temporarliy stop reading at read # :%d\n",__FUNCTION__,i);
      rdObjRegionDone =1;
      break;
    }else if(ret.first[ret.l+i-1]>b.pos){
      // fprintf(stderr,"unsorted file detected will exit new=%d new(-1)=%d\n",ret.first[ret.l+i-1],b.pos);
      fprintf(stderr,"[%s] unsorted file detected will exit new(-1)=%d new=%d,ret.l=%d i=%d\n",__FUNCTION__,ret.first[ret.l+i-1],b.pos,ret.l,i);
      exit(0);
    }
    
  }


  if(i!=nReads){
    for(int s=ret.l+i+buffed;s<ret.l+nReads;s++)
      delete [] ret.reads[s].vDat;
  }

  ret.nReads =ret.l+i;
  ret.l = ret.nReads;

}

void getPool555T(BGZF *fp,int nReads,int &isEof,sglPool &ret,int refToRead,iter_t *it,int &rdObjEof,int &rdObjRegionDone) {

  //  fprintf(stderr,"\t->[%s] buffRefid=%d\trefToRead=%d\n",__FUNCTION__,ret.bufferedRead.refID,refToRead);
  if(ret.bufferedRead.refID!=-2 && ret.bufferedRead.refID!=refToRead){
    //    fprintf(stderr,"this happens once\n");
    rdObjRegionDone =1;
    return;
  }
    
  //  fprintf(stderr,"--------\n");
  if(rdObjEof)
    return;
  //  ret.l=0;
  extern int bam_iter_read1(BGZF *fp, iter_t *iter, aRead &b);

  if((nReads+ret.l)>ret.m){
    //    fprintf(stderr,"[%s:%s()] reallocing\n",__FILE__,__FUNCTION__);
    realloc(ret,nReads+ret.l);
  }

  //int start =0;
  if(ret.bufferedRead.refID!=-2){////copy buffered
    //start = 1;
    ret.reads[ret.l] = ret.bufferedRead;//copy constructor
    ret.first[ret.l] = ret.reads[ret.l].pos;
    ret.last[ret.l] = bam_calend(ret.reads[ret.l],getCig(&ret.reads[ret.l]));
    //    fprintf(stderr,"reading buffered\n");
    ret.bufferedRead.refID = -2;
    ret.l++;
    
  }
  //start by allocating the memeory we need

  for(int i=0;i<nReads;i++){
    ///    fprintf(stderr,"allocing:%d\n",ret.l+i);
    aRead b;
    b.vDat=new uint8_t[RLEN];
    ret.reads[ret.l+i] =b;
  }

  //this is the awkward case this could cause an error in some very unlikely scenario.
  //whith the current buffer empty and the first read being a new chromosome. 
  //this is fixed good job monkey boy
  while(ret.l==0){
    //fprintf(stderr,"looping\n");
    //fprintf(stderr,"awkward\n");
    aRead &b = ret.reads[0];
    int tmp;
    if((tmp=bam_iter_read1(fp,it,b))<0){//FIXME
      if(tmp==-2){
	rdObjEof =1;
	isEof--;
      }else
	rdObjRegionDone =1;

      break;
    }
    //check that the read is == the refToread
    if(b.refID!=refToRead){
      //      buffed=1;
      ret.bufferedRead = b;
      //      fprintf(stderr,"[%s] new chromosome detected will temporarliy stop reading at read # :%d\n",__FUNCTION__,i);
      rdObjRegionDone =1;
      return;
    }

    if(b.nCig!=0){
      ret.first[ret.l] = b.pos;
      ret.last[ret.l] = bam_calend(b,getCig(&b));
      ret.l++;
      nReads--;
    }
  }
  //  fprintf(stderr,"after awkward: ret.l=%d\n",ret.l);
  /*
    now do the general loop,
    1) read an alignemnt calulate start and end pos
    check
    a) if same chromosome that a new read has a geq FINE
    b) if same but < then UNSORTED
    
  */ 
  int i;
  int tmp;
  int buffed=0;
  for( i=0;i<nReads;i++){
    //fprintf(stderr,"i+ret.l=%d\n",i+ret.l);
    //    int block_size;
    aRead &b = ret.reads[i+ret.l];

    if((tmp=bam_iter_read1(fp,it,b))<0){
      if(tmp==-2){
	rdObjEof =1;
	isEof--;
      }else
	rdObjRegionDone =1;
      break;
    }if(b.nCig==0){//only get reads with CIGAR
      i--;
      continue;
    }
    //general fine case
    if((refToRead==b.refID)&&( b.pos >= ret.first[ret.l+i-1])){
      ret.first[ret.l+i] = b.pos;
      ret.last[ret.l+i] = bam_calend(b,getCig(&b));
    }else if(b.refID>refToRead){
      buffed=1;
      ret.bufferedRead = b;
      //      fprintf(stderr,"[%s] new chromosome detected will temporarliy stop reading at read # :%d\n",__FUNCTION__,i);
      rdObjRegionDone =1;
      break;
    }else if(ret.first[ret.l+i-1]>b.pos){
      // fprintf(stderr,"unsorted file detected will exit new=%d new(-1)=%d\n",ret.first[ret.l+i-1],b.pos);
      fprintf(stderr,"[%s] unsorted file detected will exit new(-1)=%d new=%d,ret.l=%d i=%d\n",__FUNCTION__,ret.first[ret.l+i-1],b.pos,ret.l,i);
      exit(0);
    }
    
  }


  if(i!=nReads){
    for(int s=ret.l+i+buffed;s<ret.l+nReads;s++)
      delete [] ret.reads[s].vDat;
  }

  ret.nReads =ret.l+i;
  ret.l = ret.nReads;

}



int collect5(bufReader *rd,int nFiles,int &notDone,sglPool *ret,int &readNlines,int ref){
  //  fprintf(stderr,"[%s] Reading reference id=%d\n",__FUNCTION__,ref);
  int hasRead =0;

  for(int i=0;i<nFiles;i++) {
    //fprintf(stderr,"file=%d\n",i);
    if(ret[i].regionDone||ret[i].isEOF){
      ret[i].regionDone = 1;
      //      fprintf(stderr,"[%s] file=%d is Done (regdone=%d iseof=%d)\n",__FUNCTION__,i,ret[i].regionDone,ret[i].isEOF);
      continue;
    }
    if(ret[i].l>readNlines){
      //      fprintf(stderr,"file=%d ret.l=%d readNlines=%d\n",i,ret[i].l,readNlines);
      continue;
    }
    rd[i].nHasRead += readNlines;
    //getPool4(rd[i].fp,readNlines,notDone,ret[i],ref);
    //    fprintf(stderr,"\treading file=%d\n",i);
    getPool5(rd[i].fp,readNlines,notDone,ret[i],ref,&rd[i].it);
    hasRead =1;
  }
  
  if(hasRead==0){
    //    fprintf(stderr,"[%s] Nothing read due to full buffer will double buffer per file (force read)\n",__FUNCTION__);
    for(int i=0;i<nFiles;i++) {
      // fprintf(stderr,"[%s] [%d]ret.l=%d regionDone=%d\n",__FUNCTION__,i,ret[i].l,ret[i].regionDone);
      if(ret[i].regionDone)
	continue;
      rd[i].nHasRead += readNlines;
      getPool5(rd[i].fp,readNlines,notDone,ret[i],ref,&rd[i].it);
      hasRead =1;
    } 
  }
  int nDone =0;
  for(int i=0;i<nFiles;i++)
    if(ret[i].regionDone)
      nDone++;
  
  return nDone;
}



/*
  collect7 will chose a single file reads nlines, and then call getpool6, for which the last read is used as stopping point. This is avoid to many buffered reads
*/

int collect7(bufReader *rd,int nFiles,int &notDone,sglPool *ret,int &readNlines,int ref,int &pickStop) {
  //  fprintf(stderr,"[%s] Reading from reference id=%d\n",__FUNCTION__,ref);
  int usedPicker=-1;
  for(int i=0;i<nFiles;i++) {
    //    fprintf(stderr,"\nSTART of getpicker: [%s] i=%d regdone=%d\tiseof=%d\n",__FUNCTION__,i,rd[i].regionDone,rd[i].isEOF);
    if(rd[i].regionDone||rd[i].isEOF){//if file is DONE with region go to next
      rd[i].regionDone = 1;
      continue;
    }
    int pre=ret[i].l;//buffered elements before
    //    fprintf(stderr,"PRE file=%d ret.l=%d\n",i,ret[i].l);
    getPool555T(rd[i].fp,readNlines,notDone,ret[i],ref,&rd[i].it,rd[i].isEOF,rd[i].regionDone);
    //    fprintf(stderr,"POST file=%d ret.l=%d\n",i,ret[i].l);

    //first check if reading caused and end of region event to occur
    if(rd[i].regionDone||rd[0].isEOF){
      //      fprintf(stderr,"[%s] reg is done\n",__FUNCTION__);
      rd[i].regionDone = 1;
    }
    //fprintf(stderr,"[%s]pre=%d post=%d regdone=%d\n",__FUNCTION__,pre,ret[i].l,rd[i].regionDone);
    if(ret[i].l>pre){//we could read data
      //fprintf(stderr,"Setting picker\n");
      usedPicker = i;
      pickStop = ret[i].first[ret[i].l-1];
      break;
    }
  }
  //at this point we should have picked a picker

  for(int i=0;1&&i<nFiles;i++)
    //    fprintf(stderr,"ret.l = %d\n",ret[i].l);
    //  fprintf(stderr,"\n[linw=%d] pickstop=%d\tusedpicker=%d\n",__LINE__,pickStop,usedPicker);
    if(usedPicker!=-1){
      for(int i=0;i<nFiles;i++) {
	//      fprintf(stderr,"yoyoyoyoyooy i=%d rd[%d].isEOF=%d rd[%d].regionDone=%d\n",i,i,rd[i].isEOF,i,rd[i].regionDone);
	if(rd[i].isEOF || rd[i].regionDone||i==usedPicker){
	  //	fprintf(stderr,"This is causing a skipping\n");
	  continue;
	}
	//      fprintf(stderr,"i=%d regdone=%d\tiseof=%d\n",i,rd[i].regionDone,rd[i].isEOF);
      //      fprintf(stderr," getpool on i=%d\n",i);
	getPool7(rd[i].fp,readNlines,notDone,ret[i],ref,&rd[i].it,pickStop,rd[i].isEOF,rd[i].regionDone);
      } 
    }else{
      fprintf(stderr,"[%d] Usedpicker=-1 why? maybe end of region\n",__LINE__);
    //    exit(0);
    }
  int nDone =0;
  for(int i=0;i<nFiles;i++)
    if(rd[i].regionDone)
      nDone++;
  
  //  fprintf(stderr,"%s ndone=%d\n",__FUNCTION__,nDone);
  return nDone;

}


typedef struct{
  int l;//number of nodes in nodes
  int m; //possible number of of nodes
  int first;//simply a value which is equal to nodes[0].refPos;
  int last;
  node *nds;//this length can maximum be the maxlenght of a read.NOTANYMORE

}nodePool;



nodePool allocNodePool(int l){
  nodePool np;
  np.l=0;
  np.m = l;
  kroundup32(np.m);

  np.first=np.last=-1;
  np.nds = new node[np.m];
  
  return np;
}

nodePoolT allocNodePoolT(int l){
  nodePoolT np;
  np.l=0;
  np.m = l;
  kroundup32(np.m);

  np.first=np.last=-1;
  np.nds = new tNode[np.m];
  
  return np;
}


void dalloc_node(node &n){
  //  fprintf(stderr,"[%s]\n",__FUNCTION__);
  free(n.seq.s);
  free(n.qs.s);
  free(n.pos.s);
}

void dalloc_node(tNode &n){
  //  fprintf(stderr,"[%s]\n",__FUNCTION__);
  if(n.l!=0||n.m!=0){
    free(n.seq);
    free(n.qs);
    free(n.posi);
    free(n.mapQ);
    free(n.isop);
  }
}



void dalloc_nodePool (nodePool& np){
  //  fprintf(stderr,"[%s] over dn.l=%d\n",__FUNCTION__,np.l);
  for(int i=0;i<np.l;i++)
    dalloc_node(np.nds[i]);
  
}

void dalloc_nodePoolT (nodePoolT& np){
  //  fprintf(stderr,"[%s] over dn.l=%d\n",__FUNCTION__,np.l);
  for(int i=0;i<np.l;i++)
    dalloc_node(np.nds[i]);
  
}


/*
  What does this do?
  returns the number of basepairs covered by np and sgl
*/
int makeoffs(nodePool *np, sglPool *sgl){
  for(int i=0;0&&i<sgl->readIDstop;i++)
    fprintf(stderr,"sglrange[%d]\t %d\t%d\n",i,sgl->first[i],sgl->last[i]);

  int sumReg =0;
  int start=0;
  int first = np->first;
  int last = np->last +1;//because we are comparing with calc_end, which is noncontain
  //  fprintf(stderr,"np range %d\t%d  np.l=%d\n",np->firstPos,np->lastPos,np->l);


  if(np->l==0&&sgl->readIDstop!=0){//LAST PART OF CONDTIONAL WAS A MINDBUGGING BUG

    first = sgl->first[0];
    last = sgl->last[0];
    start++;
  }
  if(np->l==0&&sgl->readIDstop==0)//JESUS CHRIST
    return 0;
  for(int i=start;i<sgl->readIDstop;i++) {
    //fprintf(stderr,"pre first=%d\tlast =%d sgl.first=%d\n",first,last,sgl->first[i]);
    if(sgl->first[i]>last){
      sumReg += last-first;
      //fprintf(stderr,"SPLIT: split sumReg=%d\n",sumReg);
      first =sgl->first[i];
    }
    //    last = sgl->last[i];
    last = std::max(sgl->last[i],last);
    //    fprintf(stderr,"post first=%d\tlast =%d\n",first,last);
  }
  sumReg += last-first;
  return sumReg;
}

int makeoffsT(nodePoolT *np, sglPool *sgl){
  for(int i=0;0&&i<sgl->readIDstop;i++)
    fprintf(stderr,"sglrange[%d]\t %d\t%d\n",i,sgl->first[i],sgl->last[i]);

  int sumReg =0;
  int start=0;
  int first = np->first;
  int last = np->last +1;//because we are comparing with calc_end, which is noncontain
  //  fprintf(stderr,"np range %d\t%d  np.l=%d\n",np->firstPos,np->lastPos,np->l);


  if(np->l==0&&sgl->readIDstop!=0){//LAST PART OF CONDTIONAL WAS A MINDBUGGING BUG

    first = sgl->first[0];
    last = sgl->last[0];
    start++;
  }
  if(np->l==0&&sgl->readIDstop==0)//JESUS CHRIST
    return 0;
  for(int i=start;i<sgl->readIDstop;i++) {
    //fprintf(stderr,"pre first=%d\tlast =%d sgl.first=%d\n",first,last,sgl->first[i]);
    if(sgl->first[i]>last){
      sumReg += last-first;
      //fprintf(stderr,"SPLIT: split sumReg=%d\n",sumReg);
      first =sgl->first[i];
    }
    //    last = sgl->last[i];
    last = std::max(sgl->last[i],last);
    //    fprintf(stderr,"post first=%d\tlast =%d\n",first,last);
  }
  sumReg += last-first;
  return sumReg;
}

void mkNodePool_memsafe2_mpileup(sglPool *sgl,nodePool *np,nodePool *ret) {
  //  fprintf(stderr,"[%s] sgl.l=%d sgl.m=%d sgl.regionstop=%d\tnp.l->=%d diffs=%d\n",__FUNCTION__,sgl->l,sgl->m,sgl->readIDstop,np->l,sgl->l-sgl->readIDstop);
  int regionLen = makeoffs(np,sgl);//true covered regions
  //  fprintf(stderr,"regLen=%d\n",regionLen);
  if(regionLen==0)
    return;
  int lastSecure = sgl->lowestStart;

  //realloc if needed  
  if((regionLen)>=ret->m){
    int newMinSize = regionLen;
    kroundup32(newMinSize);
    ret->m = newMinSize;
    delete [] ret->nds;
    ret->nds = new node[ret->m];
  }
  

  node *nds = ret->nds;


  int offs = np->first;//first position from buffered
  int last = np->last+1;//because we are comparing with calc_end

  //plug in the old buffered nodes
  for(int i=0;i<np->l;i++)
    nds[np->nds[i].refPos-offs] = np->nds[i];

  //initialize new nodes for the rest of the region
  for(int i=np->l;i<regionLen;i++)
    nds[i] = initNode(UPPILE_LEN,-1,-1);

  if(np->l==0){//if we didnt have have anybuffered then
    offs = sgl->first[0];
    last = sgl->last[0];
  }

  //parse all reads
  int r;
  for( r=0;r<sgl->readIDstop;r++) {
    aRead rd = sgl->reads[r];
    //    fprintf(stderr,"r=%d\tpos=%d\n",r,rd.pos);
    if(sgl->first[r] > last){
      int diffs = (rd.pos-last);
      offs = offs + diffs;
      last = sgl->last[r];
    }else
      last = std::max(sgl->last[r],last);

    if(1&&!(rd.flag&BAM_FPROPER_PAIR)){
      fprintf(stderr,"Skipping read=%d this is not a proper read\n",r);
      continue;
    }
    char *seq =(char *) getSeq(&rd);
    char *quals =(char *) getQuals(&rd);

    int nCig = rd.nCig;
    if(nCig==0){
      fprintf(stderr,"r=%d\tnCig=%d\n",r,nCig);
      printErr();
    }
    uint32_t *cigs = getCig(&rd);
    int seq_pos =0; //position within sequence
    int wpos = rd.pos-offs;//this value is the current position assocatied with the positions at seq_pos
    //    fprintf(stderr,"[info:%d] rd.pos=%d\toffs=%d\twpos=%d\n",r,rd.pos,offs,wpos);
    //    fprintf(stderr,"\nrd->pos=%d\twpos=%d",rd->pos,wpos);
    node *tmpNode =NULL; //this is a pointer to the last node beeing modified

    //loop through read by looping through the cigar string
    for(int i=0;i<nCig;i++) {
      int opCode = cigs[i]&BAM_CIGAR_MASK; //what to do
      int opLen = cigs[i]>>BAM_CIGAR_SHIFT; //length of what to do

      if(opCode&(BAM_CINS|BAM_CDEL)){//handle insertions and deletions
	wpos--; //insertion/deletion is bound to the last position of the read
	tmpNode = &nds[wpos];
	kputc(opCode&BAM_CINS?'+':'-',&tmpNode->seq);
	kputw(opLen,&tmpNode->seq);
	
	if(opCode&BAM_CINS){
	  for(int ii=0;ii<opLen;ii++){
	    char c = bam_nt16_rev_table[bam1_seqi(seq, seq_pos)];
	    kputc(bam1_strand(&rd)? tolower(c) : toupper(c), &tmpNode->seq);
	    kputw(seq_pos+1,&tmpNode->pos);
	    kputc(',',&tmpNode->pos);
	    seq_pos++;// <- important, must be after macro
	  }
	  wpos++;
	}else {//this is the deletion part
	  for(int ii=0;ii<opLen;ii++)
	    kputc(bam1_strand(&rd)? tolower('N') : toupper('N'),&tmpNode->seq);
	  
	  wpos++;//write '*' from the next position and opLen more
	  for(int fix=wpos;wpos<fix+opLen;wpos++){

	    tmpNode = &nds[wpos];
	    tmpNode->refPos=wpos+offs;
	    tmpNode->depth ++;
	    kputc('*',&tmpNode->seq);
	    kputw(seq_pos+1, &tmpNode->pos);
	    kputc(quals[seq_pos]+33, &tmpNode->qs);
	  }
	}

      }else if(opCode&BAM_CSOFT_CLIP){
	//occurs only at the ends of the read
	if(seq_pos == 0){
	  //then we are at beginning of read and need to write mapQ
	  tmpNode = &nds[wpos];
	  tmpNode->refPos=wpos+offs;
	  kputc('^', &tmpNode->seq);
	  kputc(rd.mapQ+33, &tmpNode->seq);
	  seq_pos += opLen;
	  //	  wpos -= opLen;
	}else//we are at the end of read, then break CIGAR loop
	  break;
      }else if(opCode==BAM_CMATCH) {

	for(int fix=wpos ;wpos<(fix+opLen) ;wpos++) {
	  tmpNode =  &nds[wpos];
	  tmpNode->refPos=wpos+offs;
	  tmpNode->depth++;
	  if(seq_pos==0){
	    kputc('^', &tmpNode->seq);
	    kputc(rd.mapQ+33, &tmpNode->seq);
	  }
	  char c = bam_nt16_rev_table[bam1_seqi(seq, seq_pos)];
	  kputc(bam1_strand(&rd)? tolower(c) : toupper(c), &tmpNode->seq);
	  kputc(quals[seq_pos]+33, &tmpNode->qs);
	  kputw(seq_pos+1, &tmpNode->pos);
	  kputc(',',&tmpNode->pos);
	  tmpNode->len++;
	  seq_pos++;
	}
      }else
	printErr();//unknown CIGAR
    }
    //after end of read/parsing CIGAR always put the endline char
    //  fprintf(stderr,"printing endpileup for pos=%d\n",rd->pos);
    if(nCig!=0)
      kputc('$', &tmpNode->seq);
    delete [] rd.vDat;
  }

  //plug the reads back up //FIXME maybe do list type instead

  int miss= sgl->l-r;
  for(int i=0;i<(sgl->l-r);i++){
    sgl->reads[i] =sgl->reads[i+r];
    sgl->first[i] =sgl->first[i+r];
    sgl->last[i] =sgl->last[i+r];
  }
  sgl->l = miss;
  //copy the part not meant for printing in this round into the buffer



  int lastSecureIndex =regionLen;
  //  fprintf(stderr,"lastSecureIndex=%d\tregionLen=%d\t ret->nds.refpos=%d\n",lastSecureIndex,regionLen,ret->nds[regionLen-1].refPos);
  int tailPos = ret->nds[regionLen-1].refPos;
  if(tailPos>lastSecure)
    lastSecureIndex = regionLen-tailPos+lastSecure;
  ret->l = lastSecureIndex;

  np->l=0;
  for(int i=lastSecureIndex;i<regionLen;i++)
    np->nds[np->l++] = nds[i];


  if(np->l!=0){
    np->first = np->nds[0].refPos;
    np->last = np->nds[np->l-1].refPos;
  }

  if(ret->l!=0){
    ret->first = ret->nds[0].refPos;
    ret->last = ret->nds[ret->l-1].refPos;
    //    fprintf(stderr,"[%s] first=%d last=%d\n",__FUNCTION__,ret->first,ret->last);
  }

}
 
nodePool mkNodePool_memsafe2_mpileup2(sglPool *sgl,nodePool *np) {
  //fprintf(stderr,"[%s] sgl.l=%d sgl.m=%d sgl.regionstop=%d\tnp.l->=%d diffs=%d\n",__FUNCTION__,sgl->l,sgl->m,sgl->readIDstop,np->l,sgl->l-sgl->readIDstop);
  //  exit(0);
  int regionLen = makeoffs(np,sgl);//true covered regions
  //  fprintf(stderr,"regLen=%d\n",regionLen);
  nodePool dn;
  dn.l =0;
  if(regionLen==0)
    return dn;
  int lastSecure = sgl->lowestStart;
  
  dn = allocNodePool(regionLen);
  nodePool *ret = &dn;

  node *nds = ret->nds;


  int offs = np->first;//first position from buffered
  int last = np->last+1;//because we are comparing with calc_end

  //plug in the old buffered nodes
  for(int i=0;i<np->l;i++)
    nds[np->nds[i].refPos-offs] = np->nds[i];

  //initialize new nodes for the rest of the region
  for(int i=np->l;i<regionLen;i++)
    nds[i] = initNode(UPPILE_LEN,-1,-1);

  if(np->l==0){//if we didn't have have anybuffered then
    offs = sgl->first[0];
    last = sgl->last[0];
  }

  //parse all reads
  int r;
  for( r=0;r<sgl->readIDstop;r++) {
    aRead rd = sgl->reads[r];
    //    fprintf(stderr,"r=%d\tpos=%d\n",r,rd.pos);
    if(sgl->first[r] > last){
      int diffs = (rd.pos-last);
      offs = offs + diffs;
      last = sgl->last[r];
    }else
      last = std::max(sgl->last[r],last);

    if(0&&!(rd.flag&BAM_FPROPER_PAIR)){
      fprintf(stderr,"Skipping read=%d this is not a proper read\n",r);
      continue;
    }
    char *seq =(char *) getSeq(&rd);
    char *quals =(char *) getQuals(&rd);
    int nCig = rd.nCig;
    if(nCig==0){
      fprintf(stderr,"r=%d\tnCig=%d\n",r,nCig);
      printErr();
    }
    uint32_t *cigs = getCig(&rd);
    int seq_pos =0; //position within sequence
    int wpos = rd.pos-offs;//this value is the current position assocatied with the positions at seq_pos
    //    fprintf(stderr,"[info:%d] rd.pos=%d\toffs=%d\twpos=%d\n",r,rd.pos,offs,wpos);
    //    fprintf(stderr,"\nrd->pos=%d\twpos=%d",rd->pos,wpos);
    node *tmpNode =NULL; //this is a pointer to the last node beeing modified
    int hasInfo =0;
    //loop through read by looping through the cigar string

    for(int i=0;i<nCig;i++) {
      int opCode = cigs[i]&BAM_CIGAR_MASK; //what to do
      int opLen = cigs[i]>>BAM_CIGAR_SHIFT; //length of what to do
      //      if(opCode&BAM_CREF_SKIP)
      //	wpos += opLen;
	//fprintf(stderr,"opLen:%d\n",opLen);

      if(opCode&(BAM_CINS|BAM_CDEL)){//handle insertions and deletions
	if(i==0){ //skip indels if beginning of a read
	  tmpNode = &nds[wpos];
	  kputc('^', &tmpNode->seq);
	  kputc(rd.mapQ+33, &tmpNode->seq);
	  seq_pos += opLen;	  
	  continue;
	}
	if(hasInfo==0){
	  seq_pos += opLen;	  
	  continue;
	}
	wpos--; //insertion/deletion is bound to the last position of the read
	tmpNode = &nds[wpos];
	kputc(opCode&BAM_CINS?'+':'-',&tmpNode->seq);
	kputw(opLen,&tmpNode->seq);
	hasInfo++;
	if(opCode&BAM_CINS){
	  for(int ii=0;ii<opLen;ii++){
	    char c = bam_nt16_rev_table[bam1_seqi(seq, seq_pos)];
	    kputc(bam1_strand(&rd)? tolower(c) : toupper(c), &tmpNode->seq);
	    kputw(seq_pos+1,&tmpNode->pos);
	    kputc(',',&tmpNode->pos);
	    seq_pos++;// <- important, must be after macro
	  }
	  wpos++;
	}else {//this is the deletion part
	  for(int ii=0;ii<opLen;ii++)
	    kputc(bam1_strand(&rd)? tolower('N') : toupper('N'),&tmpNode->seq);
	  
	  wpos++;//write '*' from the next position and opLen more
	  for(int fix=wpos;wpos<fix+opLen;wpos++){

	    tmpNode = &nds[wpos];
	    tmpNode->refPos=wpos+offs;
	    tmpNode->depth ++;
	    kputc('*',&tmpNode->seq);
	    kputw(seq_pos+1, &tmpNode->pos);
	    kputc(quals[seq_pos]+33, &tmpNode->qs);
	  }
	}

      }else if(opCode&BAM_CSOFT_CLIP){
	//occurs only at the ends of the read
	if(seq_pos == 0){
	  //then we are at beginning of read and need to write mapQ
	  tmpNode = &nds[wpos];
	  tmpNode->refPos=wpos+offs;
	  kputc('^', &tmpNode->seq);
	  kputc(rd.mapQ+33, &tmpNode->seq);
	  seq_pos += opLen;
	  //	  wpos -= opLen;
	}else//we are at the end of read, then break CIGAR loop
	  break;
      }else if(opCode==BAM_CMATCH) {
	hasInfo++;
	for(int fix=wpos ;wpos<(fix+opLen) ;wpos++) {
	  tmpNode =  &nds[wpos];
	  tmpNode->refPos=wpos+offs;
	  tmpNode->depth++;
	  if(seq_pos==0){
	    kputc('^', &tmpNode->seq);
	    kputc(rd.mapQ+33, &tmpNode->seq);
	  }
	  char c = bam_nt16_rev_table[bam1_seqi(seq, seq_pos)];
	  kputc(bam1_strand(&rd)? tolower(c) : toupper(c), &tmpNode->seq);
	  kputc(quals[seq_pos]+33, &tmpNode->qs);
	  kputw(seq_pos+1, &tmpNode->pos);
	  kputc(',',&tmpNode->pos);
	  tmpNode->len++;
	  seq_pos++;
	}
      }else
	printErr();//unknown CIGAR
    }
    //after end of read/parsing CIGAR always put the endline char
    //  fprintf(stderr,"printing endpileup for pos=%d\n",rd->pos);
    if(nCig!=0)
      kputc('$', &tmpNode->seq);
    delete [] rd.vDat;
  }

  //plug the reads back up //FIXME maybe do list type instead

  int miss= sgl->l-r;
  for(int i=0;i<(sgl->l-r);i++){
    sgl->reads[i] =sgl->reads[i+r];
    sgl->first[i] =sgl->first[i+r];
    sgl->last[i] =sgl->last[i+r];
  }
  sgl->l = miss;
  //copy the part not meant for printing in this round into the buffer



  int lastSecureIndex =regionLen;
  //  fprintf(stderr,"lastSecureIndex=%d\tregionLen=%d\t ret->nds.refpos=%d\n",lastSecureIndex,regionLen,ret->nds[regionLen-1].refPos);
  int tailPos = ret->nds[regionLen-1].refPos;
  if(tailPos>lastSecure)
    lastSecureIndex = regionLen-tailPos+lastSecure;
  ret->l = lastSecureIndex;

  np->l=0;
  for(int i=lastSecureIndex;i<regionLen;i++)
    np->nds[np->l++] = nds[i];


  if(np->l!=0){
    np->first = np->nds[0].refPos;
    np->last = np->nds[np->l-1].refPos;
  }

  if(ret->l!=0){
    ret->first = ret->nds[0].refPos;
    ret->last = ret->nds[ret->l-1].refPos;
    //    fprintf(stderr,"[%s] first=%d last=%d\n",__FUNCTION__,ret->first,ret->last);
  }
  return dn;
}

int idd =0;
nodePoolT mkNodePool_memsafe2_mpileup2T(sglPool *sgl,nodePoolT *np) {
  //  fprintf(stderr,"[%s] sgl.l=%d sgl.m=%d sgl.regionstop=%d\tnp.l->=%d diffs=%d\n",__FUNCTION__,sgl->l,sgl->m,sgl->readIDstop,np->l,sgl->l-sgl->readIDstop);
  int regionLen = makeoffsT(np,sgl);//true covered regions
  //  fprintf(stderr,"regLen=%d\n",regionLen);
  nodePoolT dn;
  dn.l =0;
  if(regionLen==0)
    return dn;
  int lastSecure = sgl->lowestStart;
  
  dn = allocNodePoolT(regionLen);
  nodePoolT *ret = &dn;

  tNode *nds = ret->nds;


  int offs = np->first;//first position from buffered
  int last = np->last+1;//because we are comparing with calc_end

  //plug in the old buffered nodes
  for(int i=0;i<np->l;i++)
    nds[np->nds[i].refPos-offs] = np->nds[i];

  //initialize new nodes for the rest of the region
  for(int i=np->l;i<regionLen;i++)
    nds[i] = initNodeT(UPPILE_LEN,-1,-1);

  if(np->l==0){//if we didn't have have anybuffered then
    offs = sgl->first[0];
    last = sgl->last[0];
  }

  //parse all reads
  int r;
  actuallyReads += sgl->readIDstop;
  for( r=0;r<sgl->readIDstop;r++) {

    aRead rd = sgl->reads[r];
    int mapQ = rd.mapQ;
    //    fprintf(stderr,"mapQ=%d\n",mapQ);
    //    fprintf(stderr,"\nLength of read=%d\n",rd.l_seq);
    //    fprintf(stderr,"r=%d\tpos=%d\n",r,rd.pos);
    if(sgl->first[r] > last){
      int diffs = (rd.pos-last);
      offs = offs + diffs;
      last = sgl->last[r];
    }else
      last = std::max(sgl->last[r],last);

    if(0&&!(rd.flag&BAM_FPROPER_PAIR)){
      fprintf(stderr,"Skipping read=%d this is not a proper read\n",r);
      continue;
    }
    char *seq =(char *) getSeq(&rd);
    char *quals =(char *) getQuals(&rd);
    int nCig = rd.nCig;
    if(nCig==0){
      fprintf(stderr,"r=%d\tnCig=%d\n",r,nCig);
      printErr();
    }
    uint32_t *cigs = getCig(&rd);
    int seq_pos =0; //position within sequence
        int wpos = rd.pos-offs;//this value is the current position assocatied with the positions at seq_pos
	//    fprintf(stderr,"%d startpos=%d\n",idd++,rd.pos);
    //    fprintf(stderr,"[info:%d] rd.pos=%d\toffs=%d\twpos=%d\n",r,rd.pos,offs,wpos);
    //    fprintf(stderr,"\nrd->pos=%d\twpos=%d",rd->pos,wpos);
    tNode *tmpNode =NULL; //this is a pointer to the last node beeing modified
    int hasInfo = 0;
    //loop through read by looping through the cigar string
    for(int i=0;i<nCig;i++) {
      int opCode = cigs[i]&BAM_CIGAR_MASK; //what to do
      int opLen = cigs[i]>>BAM_CIGAR_SHIFT; //length of what to do

      if(opCode&(BAM_CINS|BAM_CDEL)){//handle insertions and deletions
	if(i==0){ //skip indels if beginning of a read
	  // tmpNode = &nds[wpos];
	  //kputc('^', &tmpNode->seq);
	  //kputc(rd.mapQ+33, &tmpNode->seq);
	  seq_pos += opLen;	  
	  continue;
	}
	if(hasInfo==0){
	  seq_pos += opLen;	  
	  continue;
	}
	wpos--; //insertion/deletion is bound to the last position of the read
	tmpNode = &nds[wpos];
	//	kputc(opCode&BAM_CINS?'+':'-',&tmpNode->seq);
	//	kputw(opLen,&tmpNode->seq);
	hasInfo++;
	if(opCode&BAM_CINS){
	  //	  fprintf(stderr,"tmpNode.refPos=%d\n",tmpNode->refPos);
	  if(tmpNode->l2 >= tmpNode->m2){
	    tmpNode->m2++;
	    kroundup32(tmpNode->m2);
	    tNode *dddd =(tNode *) malloc(tmpNode->m2*sizeof(tNode));
	    for(int ddddd=0;ddddd<tmpNode->l2;ddddd++)
	      dddd[ddddd] = tmpNode->insert[ddddd];
	    free(tmpNode->insert);
	    tmpNode->insert = dddd;
	  }
	  //	  fprintf(stderr,"tmpNode->l2=%d\n",tmpNode->l2);
	  tmpNode->insert[tmpNode->l2] = initNodeT(opLen,-9,-9);
	  for(int ii=0;ii<opLen;ii++){
	    char c = bam_nt16_rev_table[bam1_seqi(seq, seq_pos)];
	    tmpNode->insert[tmpNode->l2].seq[ii] = bam1_strand(&rd)? tolower(c) : toupper(c);
	    tmpNode->insert[tmpNode->l2].posi[ii] = seq_pos + 1;
	    tmpNode->insert[tmpNode->l2].isop[ii] =rd.l_seq- seq_pos - 1;
	    tmpNode->insert[tmpNode->l2].qs[ii] = quals[seq_pos];
	    //	    tmpNode->insert[tmpNode->l2].qs[ii] = quals[seq_pos]>mapQ?mapQ:quals[seq_pos];
	    tmpNode->insert[tmpNode->l2].mapQ[ii] = mapQ;
	    seq_pos++;// <- important, must be after macro
	  }
	  tmpNode->l2++;//incrementor!
	  //	  fprintf(stderr,"l2\t%d\n",tmpNode->l2);
	  wpos++;
	}else {//this is the deletion part
	  //	  fprintf(stderr,"opLen=%d wpos=%d\t",opLen,wpos);
	  wpos++;
	  for(int ii=0;ii<opLen;ii++){
	    tmpNode = &nds[wpos+ii];
	    tmpNode->refPos = wpos +ii + offs;
	    tmpNode->deletion ++;
	  }
	  wpos += opLen;
	  //	  fprintf(stderr,"wpos=%d\n\n",wpos);
	}

      }else if(opCode&BAM_CSOFT_CLIP){
	//occurs only at the ends of the read
	if(seq_pos == 0){
	  //then we are at beginning of read and need to write mapQ
	  tmpNode = &nds[wpos];
	  //	  tmpNode->refPos=wpos+offs;
	  //	  kputc('^', &tmpNode->seq);
	  //	  kputc(rd.mapQ+33, &tmpNode->seq);
	  seq_pos += opLen;
	  //	  wpos -= opLen;
	}else//we are at the end of read, then break CIGAR loop
	  break;
      }else if(opCode==BAM_CMATCH) {
	hasInfo++;
	for(int fix=wpos ;wpos<(fix+opLen) ;wpos++) {
	  tmpNode =  &nds[wpos];
	  tmpNode->refPos=wpos+offs;
	  if(tmpNode->l>=tmpNode->m){
	    tmpNode->m = tmpNode->m*2;
	    tmpNode->seq =(char *) realloc(tmpNode->seq,tmpNode->m);
	    tmpNode->qs =(char *) realloc(tmpNode->qs,tmpNode->m);
	    tmpNode->posi =(char *) realloc(tmpNode->posi,tmpNode->m);
	    tmpNode->isop =(char *) realloc(tmpNode->isop,tmpNode->m);
	    tmpNode->mapQ = (char *) realloc(tmpNode->mapQ,tmpNode->m);
	  }
	  

	  char c = bam_nt16_rev_table[bam1_seqi(seq, seq_pos)];
	  tmpNode->seq[tmpNode->l] = bam1_strand(&rd)? tolower(c) : toupper(c);
	  tmpNode->qs[tmpNode->l] =  quals[seq_pos];
	  //	  tmpNode->qs[tmpNode->l] =  quals[seq_pos]>mapQ?mapQ: quals[seq_pos];
	  tmpNode->posi[tmpNode->l] = seq_pos;
	  tmpNode->isop[tmpNode->l] =rd.l_seq- seq_pos-1;
	  tmpNode->mapQ[tmpNode->l] = mapQ;
	  tmpNode->l++;
	  seq_pos++;
	}
      }else
	printErr();//unknown CIGAR
    }
    //after end of read/parsing CIGAR always put the endline char
    //  fprintf(stderr,"printing endpileup for pos=%d\n",rd->pos);

    delete [] rd.vDat;
  }

  //plug the reads back up //FIXME maybe do list type instead

  int miss= sgl->l-r;
  for(int i=0;i<(sgl->l-r);i++){
    sgl->reads[i] =sgl->reads[i+r];
    sgl->first[i] =sgl->first[i+r];
    sgl->last[i] =sgl->last[i+r];
  }
  sgl->l = miss;
  //copy the part not meant for printing in this round into the buffer



  int lastSecureIndex =regionLen;
  //  fprintf(stderr,"lastSecureIndex=%d\tregionLen=%d\t ret->nds.refpos=%d\n",lastSecureIndex,regionLen,ret->nds[regionLen-1].refPos);
  int tailPos = ret->nds[regionLen-1].refPos;
  if(tailPos>lastSecure)
    lastSecureIndex = regionLen-tailPos+lastSecure;
  ret->l = lastSecureIndex;

  np->l=0;
  for(int i=lastSecureIndex;i<regionLen;i++)
    np->nds[np->l++] = nds[i];


  if(np->l!=0){
    np->first = np->nds[0].refPos;
    np->last = np->nds[np->l-1].refPos;
  }

  if(ret->l!=0){
    ret->first = ret->nds[0].refPos;
    ret->last = ret->nds[ret->l-1].refPos;
    //    fprintf(stderr,"[%s] first=%d last=%d\n",__FUNCTION__,ret->first,ret->last);
  }
  return dn;
}



/*
  old function.
  from:= min(dn);
  to:=min(max{dn[1],dn[2],...})

 */


void getMinMax2(const nodePool *dn,int nFiles,int &from,int &to){
  for(int i=0;0&&i<nFiles;i++)
    fprintf(stderr,"[%s]i=%d (%d,%d)=%d\n",__FUNCTION__,i,dn[i].first,dn[i].last,dn[i].l);

  for(int i=0;i<nFiles;i++)
    if(dn[i].l!=0){
      from = dn[i].first;
      to = dn[i].last;
      break;
    }
  
  for(int i=0;i<nFiles;i++){//might skip the 'i' used above
    if(dn[i].l==0)
      continue;
    if(dn[i].last>to)
      to = dn[i].last;
    if(dn[i].first<from)
      from = dn[i].first;
  }
}


void getMinMax2T(const nodePoolT *dn,int nFiles,int &from,int &to){
  for(int i=0;0&&i<nFiles;i++)
    fprintf(stderr,"[%s]i=%d (%d,%d)=%d\n",__FUNCTION__,i,dn[i].first,dn[i].last,dn[i].l);

  for(int i=0;i<nFiles;i++)
    if(dn[i].l!=0){
      from = dn[i].first;
      to = dn[i].last;
      break;
    }
  
  for(int i=0;i<nFiles;i++){//might skip the 'i' used above
    if(dn[i].l==0)
      continue;
    if(dn[i].last>to)
      to = dn[i].last;
    if(dn[i].first<from)
      from = dn[i].first;
  }
}


/*
  The huge matrix consiting of the tmpnode matrix is now STATIC,
This will avoid huge allocs and deallocs

 */


chunky *splitFucker3000(nodePool *dn,int nFiles) {
  static int depth_len = 0;
  static int *depth = NULL;
  static int *refPos2 = NULL;
  static node **super = NULL;
  
  if(dn==NULL){//cleanup called after end of looping through all files.
    delete [] super;
    delete [] depth;
    delete [] refPos2;
    return NULL;
  }

  int first,last;
  getMinMax2(dn,nFiles,first,last);
  //fprintf(stderr,"splitfucker (%d,%d)\n",first,last);
  int rlen = last-first+1;
  //fprintf(stderr,"\t->FORM-RO =%d,%d    rlen=%d\n",first,last,rlen);
  if(rlen>BUG_THRES){
    fprintf(stderr,"[%s] This is causing segfault\n",__FUNCTION__);
    fflush(stderr);
    exit(0);
  }
  if(rlen>=depth_len){
    fprintf(stderr,"[%s]rlen=%d reallocing old=%d\t",__FUNCTION__,rlen,depth_len);
    depth_len = rlen;
    kroundup32(depth_len);
    fprintf(stderr,"new=%d\n",depth_len);
    //cleanup old
    delete [] super;
    delete [] depth;
    delete [] refPos2;
    //make new
    super = new node*[depth_len];
    depth = new int[depth_len];
    refPos2 = new int[depth_len];
  }
  memset(depth,0,rlen*sizeof(int));
  int offs = first;
  //looping through the different samples
  for(int n=0;n<nFiles;n++){
    nodePool sm = dn[n];
    int i;
    //looping through all nodes
    for( i=0;((i<sm.l)&&(sm.nds[i].refPos <= std::min(sm.last,last) ));i++) {
      
      int posi = sm.nds[i].refPos-offs;
      if(depth[posi]==0){
	//	fprintf(stderr,"posi=%d\n",posi);
	super[posi] = new node[nFiles];
	for(int ii=0;ii<nFiles;ii++){
	  super[posi][ii] = initNode(UPPILE_LEN,posi+offs,posi);
	}
	
	refPos2[posi]=sm.nds[i].refPos;
      }
      depth[posi]++;
      super[posi][n] = sm.nds[i];
      super[posi][n].len++;
    }
    
  }
  int nnodes =0;
  for(int i=0;i<rlen;i++)
    if(depth[i]!=0)
      nnodes++;

  //  fprintf(stderr,"YYYY nnnosed=%d\n",nnodes);
  chunky *chk =new chunky;  
  chk->nd = new node*[nnodes];
  int p=0;
  int *refPos = new int[nnodes];
  for(int i=0;i<nnodes;i++)
    refPos[i] = -9;
  for(int i=0;i<rlen;i++){
    if(depth[i]!=0){
      //   fprintf(stderr,"refpos2[%d]=%d\n",i,refPos2[i]);
      refPos[p] = refPos2[i];
      chk->nd[p++] = super[i];
    }
    //   delete [] super[i];
  }
  //  fprintf(stderr,"nnodes=%d p=%d\n",nnodes,p);
  //  delete [] super;
  //  delete [] refPos2;

  chk->nSamples=nFiles;
  chk->nSites=nnodes;
  chk->refPos = refPos;
  chk->first = refPos[0];
  chk->last = refPos[nnodes-1];
  return chk;
}


typedef std::map<int,tNode *> umapT; 

chunkyT *slow_secure3000T(nodePoolT *dn,int nFiles){
  //  fprintf(stderr,"starting [%s]\n",__FUNCTION__);
  
  umapT ret;
  umapT::iterator it;
  for(int f=0;f<nFiles;f++) {
    nodePoolT sm = dn[f];
    for(int l=0;l<sm.l;l++) {
      tNode *perSite = NULL;
      int thepos =sm.nds[l].refPos;
      it=ret.find(thepos);
      if(it!=ret.end())
	perSite = it->second;
      else{
	perSite = new tNode[nFiles];
	for(int ii=0;ii<nFiles;ii++){
	  perSite[ii].l=perSite[ii].l2=perSite[ii].m=0;
	  perSite[ii].insert =NULL;
	  perSite[ii].refPos = thepos;
	}

	
	ret.insert(std::make_pair(thepos,perSite));
		
      }
      perSite[f] = sm.nds[l];
    }

  }
  //now we perform the backrool
  int nnodes = ret.size();
  int *refPos = new int [ret.size()];
  //  fprintf(stderr,"nnodes=%d\n",nnodes);
  chunkyT *chk =new chunkyT;  
  chk->nd = new tNode*[nnodes];
  int p=0;
  for(it = ret.begin();it!=ret.end();++it){
    chk->nd[p++] = it->second;
    refPos[p-1] = chk->nd[p-1][0].refPos;
  }

  chk->nSamples=nFiles;
  chk->nSites=nnodes;
  chk->refPos = refPos;
  chk->first = refPos[0];
  chk->last = refPos[nnodes-1];
  return chk;  
}


chunkyT *splitFucker3000T(nodePoolT *dn,int nFiles) {

  int *depth = NULL;
  int *refPos2 = NULL;
   tNode **super = NULL;
  
  if(dn==NULL){//cleanup called after end of looping through all files.
    return NULL;
  }
  int first,last;
  getMinMax2T(dn,nFiles,first,last);
    //fprintf(stderr,"splitfucker (%d,%d)\n",first,last);
  int rlen = last-first+1;
  //fprintf(stderr,"\t->FORM-RO =%d,%d    rlen=%d\n",first,last,rlen);
  if(rlen>BUG_THRES){
    fprintf(stderr,"[%s] Congratulations, this isn't causing segfault anymore!: rlen=%d\n",__FUNCTION__,rlen);
    fprintf(stderr,"first=%d last=%d\nWill use slow but secure version",first,last);
    fprintf(stderr,"This was not an error message, but celebration of a bugfix\n");
    return slow_secure3000T(dn,nFiles);

    fflush(stderr);
    exit(0);
    }
  super = new tNode*[rlen];
  depth = new int[rlen];
  refPos2 = new int[rlen];
  
  memset(depth,0,rlen*sizeof(int));
  int offs = first;
  //looping through the different samples
  for(int n=0;n<nFiles;n++) {
    nodePoolT sm = dn[n];
    int i;
    //looping through all nodes
    for( i=0;((i<sm.l)&&(sm.nds[i].refPos <= std::min(sm.last,last) ));i++) {
      
      int posi = sm.nds[i].refPos-offs;
      if(depth[posi]==0){
	//	fprintf(stderr,"posi=%d\n",posi);
	super[posi] = new tNode[nFiles];
	for(int ii=0;ii<nFiles;ii++){
	  super[posi][ii].l=super[posi][ii].l2=super[posi][ii].m=0;
	  super[posi][ii].insert =NULL;
	  super[posi][ii].refPos = posi+offs;
	  //super[posi][ii] = initNodeT(UPPILE_LEN,posi+offs,posi);
	}
	
	refPos2[posi]=sm.nds[i].refPos;
      }
      depth[posi]++;
      super[posi][n] = sm.nds[i];
      //      super[posi][n].len++;
    }
  }
  int nnodes =0;
  for(int i=0;i<rlen;i++)
    if(depth[i]!=0)
      nnodes++;
  
  //  fprintf(stderr,"YYYY nnnosed=%d\n",nnodes);
  chunkyT *chk =new chunkyT;  
  chk->nd = new tNode*[nnodes];
  int p=0;
  int *refPos = new int[nnodes];
  for(int i=0;i<nnodes;i++)
    refPos[i] = -9;
  for(int i=0;i<rlen;i++){
    if(depth[i]!=0){
      //   fprintf(stderr,"refpos2[%d]=%d\n",i,refPos2[i]);
      refPos[p] = refPos2[i];
      chk->nd[p++] = super[i];
    }
    //   delete [] super[i];
  }
  //  fprintf(stderr,"nnodes=%d p=%d\n",nnodes,p);
  delete [] super;
  delete [] refPos2;
  delete [] depth;
  chk->nSamples=nFiles;
  chk->nSites=nnodes;
  chk->refPos = refPos;
  chk->first = refPos[0];
  chk->last = refPos[nnodes-1];
  return chk;
  
}

chunkyT *splitFucker3000T_old(nodePoolT *dn,int nFiles) {
  static int depth_len = 0;
  static int *depth = NULL;
  static int *refPos2 = NULL;
  static tNode **super = NULL;
  
  if(dn==NULL){//cleanup called after end of looping through all files.
    delete [] super;
    delete [] depth;
    delete [] refPos2;
    return NULL;
  }

  int first,last;
  getMinMax2T(dn,nFiles,first,last);
  //fprintf(stderr,"splitfucker (%d,%d)\n",first,last);
  int rlen = last-first+1;
  //fprintf(stderr,"\t->FORM-RO =%d,%d    rlen=%d\n",first,last,rlen);
  if(rlen>BUG_THRES){
    fprintf(stderr,"[%s] This is causing segfault: rlen=%d\n",__FUNCTION__,rlen);
    fprintf(stderr,"first=%d last=%d\n",first,last);
    fflush(stderr);
    exit(0);
  }
  if(rlen>=depth_len){
    fprintf(stderr,"[%s]rlen=%d reallocing old=%d\t",__FUNCTION__,rlen,depth_len);
    depth_len = rlen;
    kroundup32(depth_len);
    fprintf(stderr,"new=%d\n",depth_len);
    //cleanup old
    delete [] super;
    delete [] depth;
    delete [] refPos2;
    //make new
    super = new tNode*[depth_len];
    depth = new int[depth_len];
    refPos2 = new int[depth_len];
  }
  memset(depth,0,rlen*sizeof(int));
  int offs = first;
  //looping through the different samples
  for(int n=0;n<nFiles;n++){
    nodePoolT sm = dn[n];
    int i;
    //looping through all nodes
    for( i=0;((i<sm.l)&&(sm.nds[i].refPos <= std::min(sm.last,last) ));i++) {
      
      int posi = sm.nds[i].refPos-offs;
      if(depth[posi]==0){
	//	fprintf(stderr,"posi=%d\n",posi);
	super[posi] = new tNode[nFiles];
	for(int ii=0;ii<nFiles;ii++){
	  super[posi][ii] = initNodeT(UPPILE_LEN,posi+offs,posi);
	}
	
	refPos2[posi]=sm.nds[i].refPos;
      }
      depth[posi]++;
      super[posi][n] = sm.nds[i];
      //      super[posi][n].len++;
    }
    
  }
  int nnodes =0;
  for(int i=0;i<rlen;i++)
    if(depth[i]!=0)
      nnodes++;

  //  fprintf(stderr,"YYYY nnnosed=%d\n",nnodes);
  chunkyT *chk =new chunkyT;  
  chk->nd = new tNode*[nnodes];
  int p=0;
  int *refPos = new int[nnodes];
  for(int i=0;i<nnodes;i++)
    refPos[i] = -9;
  for(int i=0;i<rlen;i++){
    if(depth[i]!=0){
      //   fprintf(stderr,"refpos2[%d]=%d\n",i,refPos2[i]);
      refPos[p] = refPos2[i];
      chk->nd[p++] = super[i];
    }
    //   delete [] super[i];
  }
  //  fprintf(stderr,"nnodes=%d p=%d\n",nnodes,p);
  //  delete [] super;
  //  delete [] refPos2;

  chk->nSamples=nFiles;
  chk->nSites=nnodes;
  chk->refPos = refPos;
  chk->first = refPos[0];
  chk->last = refPos[nnodes-1];
  return chk;
}



typedef struct{
  nodePool *dn;
  int nFiles;
  int regStart;
  int regStop;
  char *refName;
  chunk*(*fun)(chunky*);
}split_args;



typedef struct{
  nodePoolT *dn;
  int nFiles;
  int regStart;
  int regStop;
  char *refName;
  chunk*(*fun)(chunky*);
}split_argsT;
pthread_mutex_t mUpPile_mutex = PTHREAD_MUTEX_INITIALIZER;

void *splitfucker_thread(void *p){
  split_args *args = (split_args *)p;
  chunky *chk = splitFucker3000(args->dn,args->nFiles);
  if(args->dn!=NULL){
    chk->regStart = args->regStart;
    chk->regStop = args->regStop;
    chk->refName = args->refName;
    if(chk->refPos[0]>args->regStop){
      fprintf(stderr,"Problems with regions\n (chkFirst=%d lastReg=%d)\n",chk->refPos[0],args->regStop);
      fflush(stderr);
      exit(0);
    }
    args->fun(chk);
    for(int i=0;i<args->nFiles;i++)
      if(args->dn[i].l!=0)//this is nescesarry...
	delete [] args->dn[i].nds;
    delete [] args->dn;
    
  }
  delete args;
  pthread_mutex_unlock(&mUpPile_mutex);
  pthread_exit(NULL);
}

int print_chunkyT(chunkyT *chk,FILE *fp,char *refStr){
  //  fprintf(stderr,"[%s] nsites=%d region=(%d,%d) itrReg=(%d,%d)\n",__FUNCTION__,chk->nSites,chk->refPos[0],chk->refPos[chk->nSites-1],chk->regStart,chk->regStop);
  if(chk->refPos[0]>chk->regStop){
    fprintf(stderr,"\t->Problems with stuff\n");
    exit(0);
  }
  
  for(int s=0;s<chk->nSites;s++) {
    if(chk->refPos[s]<chk->regStart || chk->refPos[s]>chk->regStop-1 ){
      for(int i=0;i<chk->nSamples;i++)
	dalloc_node(chk->nd[s][i]);
      delete [] chk->nd[s];
      continue;
    }
    fprintf(fp,"%s\t%d",refStr,chk->refPos[s]+1);     
    
    for(int i=0;i<chk->nSamples;i++) {

      //      fprintf(stderr,"seqlen[%d,%d]=%lu\t",s,i,chk->nd[s][i].seq->l);
      if(chk->nd[s][i].l!=0){
	fprintf(fp,"\t%d\t",chk->nd[s][i].l);
	for(int l=0;l<chk->nd[s][i].l;l++)
	  fprintf(fp,"%c",chk->nd[s][i].seq[l]);
	fprintf(fp,"\t");
	for(int l=0;l<chk->nd[s][i].l;l++)
	  fprintf(fp,"%c",chk->nd[s][i].qs[l]+33);
	//	fprintf(fp,"\t");
	for(int l=0;0&&l<chk->nd[s][i].l;l++){
	  fprintf(fp,"%d:",chk->nd[s][i].posi[l]);
	}
      }else
	fprintf(fp,"\t0\t*\t*");
      dalloc_node(chk->nd[s][i]);
    }
    //    fprintf(stderr,"\n");
    fprintf(fp,"\n");
    delete [] chk->nd[s];
  }
  delete [] chk->nd;
  delete [] chk->refPos;
  delete chk;
  return 0;
}

int liketype=0;


void *splitfucker_threadT(void *p){
  fprintf(stderr,"never here\n");
  exit(0);
  split_argsT *args = (split_argsT *)p;
  chunkyT *chk = splitFucker3000T(args->dn,args->nFiles);
  if(args->dn!=NULL) {
    chk->regStart = args->regStart;
    chk->regStop = args->regStop;
    chk->refName = args->refName;
    if(chk->refPos[0]>args->regStop){
      fprintf(stderr,"Problems with regions\n (chkFirst=%d lastReg=%d)\n",chk->refPos[0],args->regStop);
      fflush(stderr);
      exit(0);
    }
    //   args->fun(chk);
    //    print_chunkyT(chk,stdout,chk->refName);
    double **lk=NULL;//gen_likesObj->call_likes(chk,NULL);
    for(int i=0;i<chk->nSites;i++)
      delete [] lk[i];
    delete [] lk;
    cleanUpChunkyT(chk);
    for(int i=0;i<args->nFiles;i++)
      if(args->dn[i].l!=0)//this is nescesarry...
	delete [] args->dn[i].nds;
    delete [] args->dn;
    
  }
  delete args;
  pthread_mutex_unlock(&mUpPile_mutex);
  pthread_exit(NULL);
}

void callBack_bambi(fcb *fff);
void splitfucker_wrapperT_noThread(nodePoolT *dn,int nFiles,int regStart,int regStop,char *refName){
  if(1){//simple hook for new function call
    //    fprintf(stderr,"making fcb \n");
    fcb *f = new fcb;
    f->dn=dn; f->nFiles = nFiles; f->regStart = regStart; f->regStop = regStop; f->refName = refName;
    callBack_bambi(f);
    // return;
  }
    
}

pthread_t thread;

void splitfucker_wrapper(nodePool *dn,int nFiles,int regStart,int regStop,char *refName,chunk*(*fun)(chunky*)){
  pthread_mutex_lock(&mUpPile_mutex);
  split_args *args = new split_args;
  args->dn=dn;
  args->nFiles=nFiles;
  args->regStart = regStart;
  args->regStop = regStop;
  args->refName = refName;
  args->fun = fun;
  

  pthread_create( &thread, NULL, splitfucker_thread, (void*) args);
  pthread_detach(thread);
}
 
void splitfucker_wrapperT(nodePoolT *dn,int nFiles,int regStart,int regStop,char *refName,chunk*(*fun)(chunky*)){

  pthread_mutex_lock(&mUpPile_mutex);
  split_argsT *args = new split_argsT;
  args->dn=dn;
  args->nFiles=nFiles;
  args->regStart = regStart;
  args->regStop = regStop;
  args->refName = refName;
  args->fun = fun;
  

  pthread_create( &thread, NULL, splitfucker_threadT, (void*) args);
  pthread_detach(thread);
}



/*
  this is a slow but memsafe version of splitfucker 2000.
*/

typedef std::map<int,node *> umap; 

chunky *slow_secure2000(nodePool *dn,int nFiles){
  fprintf(stderr,"starting [%s]\n",__FUNCTION__);
  
  umap ret;
  umap::iterator it;
  for(int f=0;f<nFiles;f++){
    nodePool sm = dn[f];
    for(int l=0;l<sm.l;l++) {
      node *perSite = NULL;
      int thepos =sm.nds[l].refPos;
      it=ret.find(thepos);
      if(it!=ret.end())
	perSite = it->second;
      else{
	perSite = new node[nFiles];
	for(int ii=0;ii<nFiles;ii++)
	  perSite[ii] = initNode(UPPILE_LEN,thepos,thepos);
	
	ret.insert(std::make_pair(thepos,perSite));
		
      }
      perSite[f] = sm.nds[l];
    }
  }
  //now we perform the backrool
  int nnodes = ret.size();
  int *refPos = new int [ret.size()];
  fprintf(stderr,"nnodes=%d\n",nnodes);
  chunky *chk =new chunky;  
  chk->nd = new node*[nnodes];
  int p=0;
  for(it = ret.begin();it!=ret.end();++it){
    chk->nd[p++] = it->second;
    refPos[p-1] = chk->nd[p-1][0].refPos;
  }

  chk->nSamples=nFiles;
  chk->nSites=nnodes;
  chk->refPos = refPos;
  chk->first = refPos[0];
  chk->last = refPos[nnodes-1];
  return chk;  
}
  



chunky *splitFucker2000(nodePool *dn,int nFiles) {
  int first,last;
  getMinMax2(dn,nFiles,first,last);
  //fprintf(stderr,"splitfucker (%d,%d)\n",first,last);
  int rlen = last-first+1; 
  //  fprintf(stderr,"\t->FORM-RO =%d,%d    rlen=%d\n",first,last,rlen);
  if(rlen>BUG_THRES){
    fprintf(stderr,"[%s.%s():%d] This is causing segfault rlen=%d\n",__FILE__,__FUNCTION__,__LINE__,rlen);
    return slow_secure2000(dn,nFiles);
    for(int i=0;i<nFiles;i++)
      if(1||dn[i].l>BUG_THRES){
	fprintf(stderr,"%d=%d\n",i,dn[i].l);
      }
    exit(0);
  }
  int depth[rlen];
  
  node **super;
  try{
    //    fprintf(stderr,"rlen=%d\n",rlen);
    fflush(stderr);
    super = new node*[rlen];
  }
  catch(char *str){
    fprintf(stderr,"rlen=%d\n",rlen);
    fprintf(stderr,"problems allocating:%s\n",str);
  }
  int *refPos2 = new int[rlen];
  memset(depth,0,sizeof(int)*rlen);
  int offs = first;
  //looping through the different samples
  for(int n=0;n<nFiles;n++){
    nodePool sm = dn[n];
    int i;
    //looping through all nodes
    for( i=0;((i<sm.l)&&(sm.nds[i].refPos <= std::min(sm.last,last) ));i++) {
      
      int posi = sm.nds[i].refPos-offs;
      if(depth[posi]==0){
	super[posi] = new node[nFiles];
	for(int ii=0;ii<nFiles;ii++){
	  super[posi][ii] = initNode(UPPILE_LEN,posi+offs,posi);
	}
	
	refPos2[posi]=sm.nds[i].refPos;
      }
      depth[posi]++;
      super[posi][n] = sm.nds[i];
      super[posi][n].len++;
    }
    
  }
  int nnodes =0;
  for(int i=0;i<rlen;i++)
    if(depth[i]!=0)
      nnodes++;

  //  fprintf(stderr,"YYYY nnnosed=%d\n",nnodes);
  chunky *chk =new chunky;  
  chk->nd = new node*[nnodes];
  int p=0;
  int *refPos = new int[nnodes];
  for(int i=0;i<nnodes;i++)
    refPos[i] = -9;
  for(int i=0;i<rlen;i++){
    if(depth[i]!=0){
      //   fprintf(stderr,"refpos2[%d]=%d\n",i,refPos2[i]);
      refPos[p] = refPos2[i];
      chk->nd[p++] = super[i];
    }
    //   delete [] super[i];
  }
  //  fprintf(stderr,"nnodes=%d p=%d\n",nnodes,p);
  delete [] super;
  delete [] refPos2;

  chk->nSamples=nFiles;
  chk->nSites=nnodes;
  chk->refPos = refPos;
  chk->first = refPos[0];
  chk->last = refPos[nnodes-1];
  return chk;
}



sglPool makePool(int l){
  sglPool ret;
  ret.l=0;
  ret.m=l;
  kroundup32(ret.m);
  if(0){
    ret.reads=(aRead *)malloc(ret.m*sizeof(aRead));
    ret.first=(int *)malloc(ret.m*sizeof(int));
    ret.last=(int *)malloc(ret.m*sizeof(int));
  }else{
    ret.reads= new aRead[ret.m];
    ret.first=new int[ret.m];
    ret.last=new int[ret.m];

  }

  ret.bufferedRead.refID = -2;
  ret.regionDone = 0;
  ret.isEOF = 0;
  //  fprintf(stderr,"allocing pool with l=%d and m=%d\n",l,ret.m);
  return ret;
}



int getSglStop5(sglPool *sglp,int nFiles,int pickStop) {
  if(0){
    fprintf(stderr,"[%s]\n",__FUNCTION__);
    for(int i=0;i<nFiles;i++){
      if(sglp[i].l!=0)
	fprintf(stderr,"ret.l=%d (%d,%d)\n",sglp[i].l,sglp[i].first[0],sglp[i].last[sglp[i].l-1]);
      else
	fprintf(stderr,"ret.l=%d (-1,-1)\n",sglp[i].l);
      for(int s=0;0&&s<sglp[i].l;s++)
	fprintf(stderr,"%d %d\n",sglp[i].first[s],sglp[i].last[s]);
      
      //    fprintf(stderr,"\n");
    }

    fprintf(stderr,"\n");
  }
  // fprintf(stderr,"pickstop=%d\n",pickStop);
  int lowestStart = pickStop ;
  if(pickStop==-1){
    lowestStart = gethighest(sglp[0].first,sglp[0].l);
    for(int i=1;i<nFiles;i++){
      if(gethighest(sglp[i].first,sglp[i].l)<lowestStart)
	lowestStart = gethighest(sglp[i].first,sglp[i].l);
      //fprintf(stderr,"i=%d\tfirstEnd=%d\n",i,lowestStart);
    }
  }

  lowestStart--;
  //fprintf(stderr,"loweststart=%d\n",lowestStart);
  /*
    now we looptrhough each readpool, and find 2 values.
    1) the readnumber of the last read to be included for uppiling,this
    2) the lastposition for which we know that the uppiling is complete
  */
  int tmp =0;
  for(int i=0;i<nFiles;i++){
    sglp[i].lowestStart = lowestStart;
    int j;
    for( j=0;j<sglp[i].l;j++)
      if(sglp[i].first[j]>lowestStart)
	break;
    sglp[i].readIDstop=j;
    tmp += j;
  }
  return tmp;
}


/*
  this function just returns the highest position along all buffered nodepools and readpools
 */
void getMaxMax2(sglPool *sglp,int nFiles,nodePool *nps){
  //
  if(0){
    fprintf(stderr,"[%s].nFiles=%d\n",__FUNCTION__,nFiles);
    for(int i=0;i<nFiles;i++){
      fprintf(stderr,"%d=sglp.l=%d\n",i,sglp[i].l);
    }
  }

  //this sholdnt be called lowestStart but rather hightestLast
  int lowestStart = 0;
  //  fprintf(stderr,"[%s] i=0\tfirstEnd=%d\n",__FUNCTION__,lowestStart);

  for(int i=0;i<nFiles;i++){
    if(gethighest(sglp[i].last,sglp[i].l)>lowestStart)
      lowestStart = gethighest(sglp[i].last,sglp[i].l);
    if(nps[i].last>lowestStart)
      lowestStart = nps[i].last;
    //fprintf(stderr,"i=%d\tfirstEnd=%d\n",i,lowestStart);
  }
  //  fprintf(stderr,"[%s] \thightestlastpostion=%d\n",__FUNCTION__,lowestStart);
  for(int i=0;i<nFiles;i++){
    sglp[i].lowestStart = lowestStart;
    sglp[i].readIDstop=sglp[i].l;
  }

}

void getMaxMax2T(sglPool *sglp,int nFiles,nodePoolT *nps){
  //
  if(0){
    fprintf(stderr,"[%s].nFiles=%d\n",__FUNCTION__,nFiles);
    for(int i=0;i<nFiles;i++){
      fprintf(stderr,"%d=sglp.l=%d\n",i,sglp[i].l);
    }
  }

  //this sholdnt be called lowestStart but rather hightestLast
  int lowestStart = 0;
  //  fprintf(stderr,"[%s] i=0\tfirstEnd=%d\n",__FUNCTION__,lowestStart);

  for(int i=0;i<nFiles;i++){
    if(gethighest(sglp[i].last,sglp[i].l)>lowestStart)
      lowestStart = gethighest(sglp[i].last,sglp[i].l);
    if(nps[i].last>lowestStart)
      lowestStart = nps[i].last;
    //fprintf(stderr,"i=%d\tfirstEnd=%d\n",i,lowestStart);
  }
  //  fprintf(stderr,"[%s] \thightestlastpostion=%d\n",__FUNCTION__,lowestStart);
  for(int i=0;i<nFiles;i++){
    sglp[i].lowestStart = lowestStart;
    sglp[i].readIDstop=sglp[i].l;
  }

}



void *setIterator1(void *args){
  bufReader *rd = (bufReader *) args;
  free(rd->it.off);
  //  fprintf(stderr,"[%s ] %s\n",__FUNCTION__,rd->regions[rd->itrPos]);
  int start,stop,ref;
  start = rd->regions[rd->itrPos].start;
  stop = rd->regions[rd->itrPos].stop;
  ref = rd->regions[rd->itrPos].refID;
  getOffsets(rd->fname,rd->hd,rd->it,ref,start,stop);//leak
  return EXIT_SUCCESS;
}


 void setIterator1_thread(void *args,int index){
  bufReader *rds = (bufReader *) args;
  bufReader *rd = &rds[index];
  free(rd->it.off);
  int start,stop,ref;
  start = rd->regions[rd->itrPos].start;
  stop = rd->regions[rd->itrPos].stop;
  ref = rd->regions[rd->itrPos].refID;
  getOffsets(rd->fname,rd->hd,rd->it,ref,start,stop);


  //  fprintf(stderr,"[%s ] %s\n",__FUNCTION__,rd->regions[rd->itrPos]);
  //  getOffsets(strdup(rd->regions[rd->itrPos]),rd->fname,rd->hd,rd->it);//leak
}


void setIterators(bufReader *rd,int itrPos,int nFiles,int nThreads){
  for(int i=0;1&&i<nFiles;i++){
    rd[i].itrPos=itrPos;
    if(nThreads==0)
      setIterator1(&rd[i]);
  }
  if(nThreads>0){
    threadpool *tp = init_threadpool(nThreads,setIterator1_thread,nFiles,1);
    threadpool_iter(tp,rd,nFiles);
    wait_kill(tp);
    tp = NULL;
  }
}

//type=0 -> memsecure
//type=1 -> not memsafe, maybe faster

int motherUppile_new3_mpileup(int type,int nThreads,bufReader *rd,int NLINES,int nFiles,chunk*(*fun)(chunky*)) {
  //  fprintf(stderr,"\t[%s]-> type=%d\n",__FUNCTION__, type);
  nodePool *nps = new nodePool[nFiles];// <- buffered nodes
  nodePool *dn = new nodePool[nFiles];//these contains the nodes that should to be printed, aligned to same positions
  sglPool *sglp= new sglPool[nFiles];

  //  exit(0);
  for(int i=0;i<nFiles;i++){
    nps[i] = allocNodePool(MAX_SEQ_LEN);
    dn[i] = allocNodePool(500);
    sglp[i] = makePool(NLINES);
  }

  //apparently a counter keeping track of which region we are currently in(lean to document your code thorfinn...)
  int itrPos = -1;
  int sumEof = 0;//sumof readerobject that has eof


  while( SIG_COND){

    int theRef;
    int regStart=-1;
    int regStop =270000000;
    int notDone = nFiles;
    sumEof =0;
    //reset the done region flag, while checking that any file still has data
    for(int i=0;i<nFiles;i++){
      sumEof += rd[i].isEOF;
      rd[i].regionDone =0;
    }
    for(int i=0;0&&i<nFiles;i++)
      fprintf(stderr,"[%s] i=%d eof=%d regionDone=%d\n",__FUNCTION__,i,rd[i].isEOF,rd[i].regionDone);


    //break loop if all files are done
    if(sumEof==nFiles)
      break;

    if(rd[0].regions.size()!=0) {
      if(itrPos+1==(int)rd[0].regions.size()){
	//	fprintf(stderr,"\nNo more regions to parse(from region list)\n");     
	fflush(stderr);
	break;
      }else {//important to typecast
	itrPos++;
	//	fprintf(stderr,"%d/%lu ",itrPos+1,rd[0].regions.size());
	setIterators(rd,itrPos,nFiles,nThreads);
	theRef = rd[0].it.tid;
	
	//validate we have offsets;
	int gotData = 0;
	for(int i=0;i<nFiles;i++)
	  if(rd[0].it.off!=NULL){
	    gotData =1;
	    break;
	  }
	if(gotData==0){
	  regs rrr = rd[0].regions[itrPos];
	  fprintf(stderr,"\t[%s] skipping region=%d,%d,%d (noData)\n",__FUNCTION__,rrr.refID,rrr.start,rrr.stop);
	  continue;
	}
	//int tmpRef=rd[0].regions[itrPos].refID;
	regStart = rd[0].regions[itrPos].start;
	regStop = rd[0].regions[itrPos].stop;

	//reset eof and region donepointers.
	for(int i=0;i<nFiles;i++){
	  rd[i].isEOF = 0;
	  rd[i].regionDone =0;
	  sglp[i].l =0;
	  sglp[i].bufferedRead.refID=-2;
	  //	  nps[i].l=0;
	}
      }
    }else{
      int minRef = rd[0].hd->n_ref;
      for(int i=0;i<nFiles;i++)
	if(sglp[i].bufferedRead.refID<minRef)
	  minRef = sglp[i].bufferedRead.refID;
      if(minRef==-2)
	theRef = 0;
      else
	theRef = minRef;
    }

    //as long as we have unfinished files
    while(notDone&& SIG_COND) { //main loop will loop through chromosome/region

      int pickStop=-1;
      int doFlush;
      if(type==0)
	doFlush = (collect5(rd,nFiles,notDone,sglp,NLINES,theRef)==nFiles)?1:0 ;
      else
	doFlush = (collect7(rd,nFiles,notDone,sglp,NLINES,theRef,pickStop)==nFiles)?1:0 ;
      //      fprintf(stderr,"[%s] pickStop=%d\n",__FUNCTION__,pickStop);
      if(0){
	fprintf(stderr,"\n");
	for(int i=0;i<nFiles;i++){
	  fprintf(stderr,"[%s] sgl[%d].l=%d (%d,%d)\n",__FUNCTION__,i,sglp[i].l,sglp[i].first[0],sglp[i].last[sglp[i].l-1]);
	}
	for(int i=0;i<nFiles;i++){
	  fprintf(stderr,"[%s] i=%d nps.l=%d",__FUNCTION__,i,nps[i].l);
	  if(nps[i].l!=0)
	    fprintf(stderr,"(%d,%d)\n",nps[i].first,nps[i].last);
	  else
	    fprintf(stderr,"\n");
	}
      }
      //prepare uppiling for all data, if end of chromosome or all files are EOF
      //fprintf(stderr,"doFlush=%d\tnotDone=%d\n",doFlush,notDone);
      if(doFlush||notDone==0){
	//	fprintf(stderr,"[%s]probs with stuff maybe last chunk Will flush all remaining reads\n",__FUNCTION__);
	getMaxMax2(sglp,nFiles,nps);
      }else{
	//getSglStop returns the sum of reads to parse.
	int hasData = getSglStop5(sglp,nFiles,pickStop);
	//	fprintf(stderr,"hasDAta=%d\n",hasData);
	if(hasData==0&&notDone!=0){
	  //	  fprintf(stderr,"Triggering extra filereading (before uppiling) reads:notDone=%d SIG_COND=%d\n",notDone,SIG_COND);
	  continue;
	}
      }
      
      //make uppile nodes
      for(int i=0;i<nFiles;i++)
	mkNodePool_memsafe2_mpileup(&sglp[i],&nps[i],&dn[i]);
      if(0){
	for(int i=0;i<nFiles;i++)
	  fprintf(stderr,"l=%d first=%d last=%d\n",dn[i].l,dn[i].first,dn[i].last);

      }

      //fprintf(stderr,"[%s] after memsafe2\n",__FUNCTION__);
      
      //simple sanity check for validating that we have indeed something to print.
      int tmpSum = 0;
      for(int i=0;i<nFiles;i++){
	//	fprintf(stderr,"i=%d\n",i);
	tmpSum += dn[i].l;
	//	printIter(rd[i].it,stderr);
	if(0&& dn[i].l>10000){
	  fprintf(stderr,"seqgfaul %d=%d\n",i,dn[i].l);
	  fprintf(stderr,"fname=%s\n",rd[i].fname);
	  printIter(rd[i].it,stderr);
	  exit(0);
	}
      }
      if(tmpSum==0){
	fprintf(stderr,"No data for chromoId=%d chromoname=%s\n",theRef,rd[0].hd->name[theRef]);
	fprintf(stderr,"This could either indicate that there really is no data for this chromosome\n");
	fprintf(stderr,"Or it could be problem with this program\n");
	break;
      }

      //merge the perFile upnodes.FIXME for large regions (with gaps) this will allocate to much...
      chunky *chk =splitFucker2000(dn,nFiles);
      if(chk->refPos[0]>regStop){
	fprintf(stderr,"Problems with regions\n (chkFirst=%d lastReg=%d)\n",chk->refPos[0],regStop);
	break;
	//	exit(0);
      }
      chk->regStart = regStart;
      chk->regStop = regStop;
      chk->refName = rd[0].hd->name[theRef];
      fun(chk);
      //      printChunky2(chk,stdout,rd[0].hd->name[theRef]);
      for(int i=0;1&&i<nFiles;i++)
	dn[i].l =0;

      //if we flush, its due to end of chr/region or end of files
      if(doFlush)
	break;
      //      SIG_COND=0;
    }

  }
  //clean up

  for(int i=0;i<nFiles;i++){
    dalloc(rd[i].it.dasIndex);

    //the nodes are cleaned in the printing, only free the array
    delete [] dn[i].nds;
    dalloc_nodePool(nps[i]);
    delete [] nps[i].nds;
    dalloc(sglp[i]);
  }

  
  //  fprintf(stderr,"notDone=%d\tstopper=%d\n",notDone,stopper);
  delete [] dn;
  delete [] nps;
  delete [] sglp;
  return 0;
}



int motherUppile_new3_mpileup_doneNodesReinit(int type,int nThreads,bufReader *rd,int NLINES,int nFiles,chunk*(*fun)(chunky*)) {
  //  fprintf(stderr,"[%s] reading type=%d \n",__FUNCTION__,type);

  nodePool *nps = new nodePool[nFiles];// <- buffered nodes
  sglPool *sglp= new sglPool[nFiles];

  //  exit(0);
  for(int i=0;i<nFiles;i++){
    nps[i] = allocNodePool(MAX_SEQ_LEN);
    sglp[i] = makePool(NLINES);
  }

  //apparently a counter keeping track of which region we are currently in(lean to document your code thorfinn...)
  int itrPos = -1;
  int sumEof = 0;//sumof readerobject that has eof


  while( SIG_COND){
    int theRef;
    int regStart=-1;
    int regStop =270000000;
    int notDone = nFiles;
    sumEof =0;
    //reset the done region flag, while checking that any file still has data
    for(int i=0;i<nFiles;i++){
      sumEof += rd[i].isEOF;
      rd[i].regionDone =0;
    }
    for(int i=0;0&&i<nFiles;i++)
      fprintf(stderr,"[%s] i=%d eof=%d regionDone=%d\n",__FUNCTION__,i,rd[i].isEOF,rd[i].regionDone);


    //break loop if all files are done
    if(sumEof==nFiles)
      break;

    if(rd[0].regions.size()!=0){
      if(itrPos+1==(int)rd[0].regions.size()){
	//	fprintf(stderr,"\nNo more regions to parse(from region list)\n");     
	fflush(stderr);
	break;
      }else {//important to typecast
	itrPos++;
	fprintf(stderr,"%d/%lu ",itrPos+1,rd[0].regions.size());
	setIterators(rd,itrPos,nFiles,nThreads);
	theRef = rd[0].it.tid;
	
	//validate we have offsets;
	int gotData = 0;
	for(int i=0;i<nFiles;i++)
	  if(rd[0].it.off!=NULL){
	    gotData =1;
	    break;
	  }
	if(gotData==0){
	  regs rrr = rd[0].regions[itrPos];
	  fprintf(stderr,"\t[%s] skipping region=%d,%d,%d (noData)\n",__FUNCTION__,rrr.refID,rrr.start,rrr.stop);
	  continue;
	}
	//int tmpRef=rd[0].regions[itrPos].refID;
	regStart = rd[0].regions[itrPos].start;
	regStop = rd[0].regions[itrPos].stop;

	//reset eof and region donepointers.
	for(int i=0;i<nFiles;i++){
	  rd[i].isEOF = 0;
	  rd[i].regionDone =0;
	  sglp[i].l =0;
	  sglp[i].bufferedRead.refID=-2;
	}
      }
    }else{
      int minRef = rd[0].hd->n_ref;
      for(int i=0;i<nFiles;i++)
	if(sglp[i].bufferedRead.refID<minRef)
	  minRef = sglp[i].bufferedRead.refID;
      if(minRef==-2)
	theRef = 0;
      else
	theRef = minRef;
    }

    //as long as we have unfinished files
    while(notDone&& SIG_COND) { //main loop will loop through chromosome/region
      int pickStop=-1;
      int doFlush;
      if(type==0)
	doFlush = (collect5(rd,nFiles,notDone,sglp,NLINES,theRef)==nFiles)?1:0 ;
      else
	doFlush = (collect7(rd,nFiles,notDone,sglp,NLINES,theRef,pickStop)==nFiles)?1:0 ;
      //      fprintf(stderr,"[%s] pickStop=%d\n",__FUNCTION__,pickStop);
      if(0){
	fprintf(stderr,"\n");
	for(int i=0;i<nFiles;i++){
	  fprintf(stderr,"[%s] sgl[%d].l=%d (%d,%d)\n",__FUNCTION__,i,sglp[i].l,sglp[i].first[0],sglp[i].last[sglp[i].l-1]);
	}
	for(int i=0;i<nFiles;i++){
	  fprintf(stderr,"[%s] i=%d nps.l=%d",__FUNCTION__,i,nps[i].l);
	  if(nps[i].l!=0)
	    fprintf(stderr,"(%d,%d)\n",nps[i].first,nps[i].last);
	  else
	    fprintf(stderr,"\n");
	}
      }
      //prepare uppiling for all data, if end of chromosome or all files are EOF
      //fprintf(stderr,"doFlush=%d\tnotDone=%d\n",doFlush,notDone);
      if(doFlush||notDone==0){
	//	fprintf(stderr,"[%s]probs with stuff maybe last chunk Will flush all remaining reads\n",__FUNCTION__);
	getMaxMax2(sglp,nFiles,nps);
      }else{
	//getSglStop returns the sum of reads to parse.
	int hasData = getSglStop5(sglp,nFiles,pickStop);
	//	fprintf(stderr,"hasDAta=%d\n",hasData);
	if(hasData==0&&notDone!=0){
	  //	  fprintf(stderr,"Triggering extra filereading (before uppiling) reads:notDone=%d SIG_COND=%d\n",notDone,SIG_COND);
	  continue;
	}
      }
      
      //make uppile nodes
      nodePool dn[nFiles];
      for(int i=0;i<nFiles;i++){
	dn[i] = mkNodePool_memsafe2_mpileup2(&sglp[i],&nps[i]);
	//	fprintf(stderr,"AFTER mknodepool[%d] first=%d last=%d\n",i,dn[i].first,dn[i].last);
      }
      if(0){
	for(int i=0;i<nFiles;i++)
	  fprintf(stderr,"l=%d first=%d last=%d\n",dn[i].l,dn[i].first,dn[i].last);

      }

      //fprintf(stderr,"[%s] after memsafe2\n",__FUNCTION__);
      
      //simple sanity check for validating that we have indeed something to print.
      int tmpSum = 0;
      for(int i=0;i<nFiles;i++){
	//	fprintf(stderr,"i=%d\n",i);
	tmpSum += dn[i].l;
	//	printIter(rd[i].it,stderr);
	if(0&& dn[i].l>10000){
	  fprintf(stderr,"seqgfaul %d=%d\n",i,dn[i].l);
	  fprintf(stderr,"fname=%s\n",rd[i].fname);
	  printIter(rd[i].it,stderr);
	  exit(0);
	}
      }
      if(tmpSum==0){
	fprintf(stderr,"No data for chromoId=%d chromoname=%s\n",theRef,rd[0].hd->name[theRef]);
	fprintf(stderr,"This could either indicate that there really is no data for this chromosome\n");
	fprintf(stderr,"Or it could be problem with this program\n");
	break;
      }

      //merge the perFile upnodes.FIXME for large regions (with gaps) this will allocate to much...
      chunky *chk =splitFucker2000(dn,nFiles);
      if(chk->refPos[0]>regStop){
	fprintf(stderr,"Problems with regions\n (chkFirst=%d lastReg=%d)\n",chk->refPos[0],regStop);
	break;
	//	exit(0);
      }
      chk->regStart = regStart;
      chk->regStop = regStop;
      chk->refName = rd[0].hd->name[theRef];
      fun(chk);
    
      for(int i=0;1&&i<nFiles;i++){
	if(dn[i].l!=0)
	  delete [] dn[i].nds;
      }
      //if we flush, its due to end of chr/region or end of files
      if(doFlush)
	break;
    }

  }
  //clean up
  for(int i=0;1&&i<nFiles;i++){
    dalloc(rd[i].it.dasIndex);
    dalloc_nodePool(nps[i]);
    delete [] nps[i].nds;
    dalloc(sglp[i]);
  }

  delete [] nps;
  delete [] sglp;
  return 0;
}

int motherUppile_new3_mpileup_doneNodesReinit_respawn(int type,int nThreads,bufReader *rd,int NLINES,int nFiles,chunk*(*fun)(chunky*)) {
  fprintf(stderr,"[%s] \t-> type=%d\n",__FUNCTION__,type);
  nodePool *nps = new nodePool[nFiles];// <- buffered nodes
  sglPool *sglp= new sglPool[nFiles];

  //  exit(0);
  for(int i=0;i<nFiles;i++){
    nps[i] = allocNodePool(MAX_SEQ_LEN);
    sglp[i] = makePool(NLINES);
  }

  //apparently a counter keeping track of which region we are currently in(lean to document your code thorfinn...)
  int itrPos = -1;
  int sumEof = 0;//sumof readerobject that has eof


  while( SIG_COND){
    int theRef;
    int regStart=-1;
    int regStop =270000000;
    int notDone = nFiles;
    sumEof =0;
    //reset the done region flag, while checking that any file still has data
    for(int i=0;i<nFiles;i++){
      sumEof += rd[i].isEOF;
      rd[i].regionDone =0;
    }
    for(int i=0;0&&i<nFiles;i++)
      fprintf(stderr,"[%s] i=%d eof=%d regionDone=%d\n",__FUNCTION__,i,rd[i].isEOF,rd[i].regionDone);


    //break loop if all files are done
    if(sumEof==nFiles)
      break;

    if(rd[0].regions.size()!=0){
      if(itrPos+1==(int)rd[0].regions.size()){
	//	fprintf(stderr,"\nNo more regions to parse(from region list)\n");     
	fflush(stderr);
	break;
      }else {//important to typecast
	itrPos++;
	fprintf(stderr,"%d/%lu ",itrPos+1,rd[0].regions.size());
	setIterators(rd,itrPos,nFiles,nThreads);
	theRef = rd[0].it.tid;
	
	//validate we have offsets;
	int gotData = 0;
	for(int i=0;i<nFiles;i++)
	  if(rd[0].it.off!=NULL){
	    gotData =1;
	    break;
	  }
	if(gotData==0){
	  regs rrr = rd[0].regions[itrPos];
	  fprintf(stderr,"\t[%s] skipping region=%d,%d,%d (noData)\n",__FUNCTION__,rrr.refID,rrr.start,rrr.stop);
	  continue;
	}
	//int tmpRef=rd[0].regions[itrPos].refID;
	regStart = rd[0].regions[itrPos].start;
	regStop = rd[0].regions[itrPos].stop;

	//reset eof and region donepointers.
	for(int i=0;i<nFiles;i++){
	  rd[i].isEOF = 0;
	  rd[i].regionDone =0;
	  sglp[i].l =0;
	  sglp[i].bufferedRead.refID=-2;
	}
      }
    }else{
      int minRef = rd[0].hd->n_ref;
      for(int i=0;i<nFiles;i++)
	if(sglp[i].bufferedRead.refID<minRef)
	  minRef = sglp[i].bufferedRead.refID;
      if(minRef==-2)
	theRef = 0;
      else
	theRef = minRef;
    }

    //as long as we have unfinished files
    while(notDone&& SIG_COND) { //main loop will loop through chromosome/region
      int pickStop=-1;
      int doFlush;
      if(type==0)
	doFlush = (collect5(rd,nFiles,notDone,sglp,NLINES,theRef)==nFiles)?1:0 ;
      else
	doFlush = (collect7(rd,nFiles,notDone,sglp,NLINES,theRef,pickStop)==nFiles)?1:0 ;
      //      fprintf(stderr,"[%s] pickStop=%d\n",__FUNCTION__,pickStop);
      if(0){
	fprintf(stderr,"\n");
	for(int i=0;i<nFiles;i++){
	  fprintf(stderr,"[%s] sgl[%d].l=%d (%d,%d)\n",__FUNCTION__,i,sglp[i].l,sglp[i].first[0],sglp[i].last[sglp[i].l-1]);
	}
	for(int i=0;i<nFiles;i++){
	  fprintf(stderr,"[%s] i=%d nps.l=%d",__FUNCTION__,i,nps[i].l);
	  if(nps[i].l!=0)
	    fprintf(stderr,"(%d,%d)\n",nps[i].first,nps[i].last);
	  else
	    fprintf(stderr,"\n");
	}
      }
      //prepare uppiling for all data, if end of chromosome or all files are EOF
      //fprintf(stderr,"doFlush=%d\tnotDone=%d\n",doFlush,notDone);
      if(doFlush||notDone==0){
	//	fprintf(stderr,"[%s]probs with stuff maybe last chunk Will flush all remaining reads\n",__FUNCTION__);
	getMaxMax2(sglp,nFiles,nps);
      }else{
	//getSglStop returns the sum of reads to parse.
	int hasData = getSglStop5(sglp,nFiles,pickStop);
	//	fprintf(stderr,"hasDAta=%d\n",hasData);
	if(hasData==0&&notDone!=0){
	  //	  fprintf(stderr,"Triggering extra filereading (before uppiling) reads:notDone=%d SIG_COND=%d\n",notDone,SIG_COND);
	  continue;
	}
      }
      
      //make uppile nodes
      nodePool *dn = new nodePool[nFiles];
      for(int i=0;i<nFiles;i++){
	dn[i] = mkNodePool_memsafe2_mpileup2(&sglp[i],&nps[i]);
	//	fprintf(stderr,"AFTER mknodepool[%d] first=%d last=%d\n",i,dn[i].first,dn[i].last);
      }
      if(0){
	for(int i=0;i<nFiles;i++)
	  fprintf(stderr,"l=%d first=%d last=%d\n",dn[i].l,dn[i].first,dn[i].last);

      }

      //fprintf(stderr,"[%s] after memsafe2\n",__FUNCTION__);
      
      //simple sanity check for validating that we have indeed something to print.
      int tmpSum = 0;
      for(int i=0;i<nFiles;i++){
	//	fprintf(stderr,"i=%d\n",i);
	tmpSum += dn[i].l;
	//	printIter(rd[i].it,stderr);
	if(0&& dn[i].l>10000){
	  fprintf(stderr,"seqgfaul %d=%d\n",i,dn[i].l);
	  fprintf(stderr,"fname=%s\n",rd[i].fname);
	  printIter(rd[i].it,stderr);
	  exit(0);
	}
      }
      if(tmpSum==0){
	fprintf(stderr,"No data for chromoId=%d chromoname=%s\n",theRef,rd[0].hd->name[theRef]);
	fprintf(stderr,"This could either indicate that there really is no data for this chromosome\n");
	fprintf(stderr,"Or it could be problem with this program\n");
	break;
      }

      //merge the perFile upnodes.FIXME for large regions (with gaps) this will allocate to much...
      //spawns threads
      splitfucker_wrapper(dn,nFiles,regStart,regStop,rd[0].hd->name[theRef],fun);
      
  
      //if we flush, its due to end of chr/region or end of files
      if(doFlush)
	break;
    }

  }
  //clean up
  for(int i=0;1&&i<nFiles;i++){
    dalloc(rd[i].it.dasIndex);
    dalloc_nodePool(nps[i]);
    delete [] nps[i].nds;
    dalloc(sglp[i]);
  }
  splitfucker_wrapper(NULL,0,0,0,NULL,NULL);
  delete [] nps;
  delete [] sglp;
  return 0;
}



int doThread = 0;

int faster(int type,int nThreads,bufReader *rd,int NLINES,int nFiles,chunk*(*fun)(chunky*)) {
  //  fprintf(stderr,"[%s] ",__FUNCTION__);
  pthread_mutex_lock(&mUpPile_mutex);//just to make sure, its okey to clean up
  //  fprintf(stderr,"\t-> type=%d\n",type);
  nodePoolT *nps = new nodePoolT[nFiles];// <- buffered nodes
  sglPool *sglp= new sglPool[nFiles];

  //  exit(0);
  for(int i=0;i<nFiles;i++){
    nps[i] = allocNodePoolT(MAX_SEQ_LEN);
    sglp[i] = makePool(NLINES);
  }

  //apparently a counter keeping track of which region we are currently in(lean to document your code thorfinn...)
  int itrPos = -1;
  int sumEof = 0;//sumof readerobject that has eof


  while( SIG_COND) {

    int theRef;
    int regStart=-1;
    int regStop =270000000;
    int notDone = nFiles;
    sumEof =0;
    //reset the done region flag, while checking that any file still has data
    for(int i=0;i<nFiles;i++){
      sumEof += rd[i].isEOF;
      rd[i].regionDone =0;
    }
    //    for(int i=0;0&&i<nFiles;i++)
    //  fprintf(stderr,"[%s] i=%d eof=%d regionDone=%d\n",__FUNCTION__,i,rd[i].isEOF,rd[i].regionDone);


    //break loop if all files are done ANDERS FIX BUG
    if(sumEof==nFiles &&rd[0].regions.size()==0)
      break;

    //    fprintf(stderr,"regions.zie()=%lu\n",rd[0].regions.size());
    //   exit(0);
    if(rd[0].regions.size()!=0) {
      if(itrPos+1==(int)rd[0].regions.size()){
	//	fprintf(stderr,"\nNo more regions to parse(from region list)\n");     
	//	fflush(stderr);
	break;
      }else {//important to typecast
	itrPos++;
	//	fprintf(stderr,"%d/%lu ",itrPos+1,rd[0].regions.size());
	setIterators(rd,itrPos,nFiles,nThreads);
	theRef = rd[0].it.tid;
	
	//validate we have offsets;
	int gotData = 0;
	for(int i=0;i<nFiles;i++)
	  if(rd[0].it.off!=NULL){
	    gotData =1;
	    break;
	  }
	if(gotData==0){
	  regs rrr = rd[0].regions[itrPos];
	  fprintf(stderr,"\t[%s] skipping region=%d,%d,%d (noData)\n",__FUNCTION__,rrr.refID,rrr.start,rrr.stop);
	  continue;
	}
	//	int tmpRef=rd[0].regions[itrPos].refID;
	//	fprintf(stderr,"tmpREF in dasshi=%d\n",tmpRef);
	regStart = rd[0].regions[itrPos].start;
	regStop = rd[0].regions[itrPos].stop;

	//reset eof and region donepointers.
	for(int i=0;i<nFiles;i++){
	  rd[i].isEOF = 0;
	  rd[i].regionDone =0;
	  sglp[i].l =0;
	  sglp[i].bufferedRead.refID=-2;
	}
      }
    }else{
      int minRef = rd[0].hd->n_ref;
      for(int i=0;i<nFiles;i++)
	if(sglp[i].bufferedRead.refID<minRef)
	  minRef = sglp[i].bufferedRead.refID;
      if(minRef==-2)
	theRef = 0;
      else
	theRef = minRef;
    }

    //    fprintf(stderr,"theREf=%d\n",theRef);
    //exit(0);
    //as long as we have unfinished files
    while(notDone&& SIG_COND) { //main loop will loop through chromosome/region
      //      fprintf(stderr,"theREf=%d in the while type=%d\n",theRef,type);
      int pickStop=-1;
      int doFlush;
      //      fprintf(stderr,"before collect: theREf ref=%d sglp.l=%d\n",theRef,sglp[0].l);
      if(type==0)
	doFlush = (collect5(rd,nFiles,notDone,sglp,NLINES,theRef)==nFiles)?1:0 ;
      else
	doFlush = (collect7(rd,nFiles,notDone,sglp,NLINES,theRef,pickStop)==nFiles)?1:0 ;
      //      fprintf(stderr,"[%s] pickStop=%d\n",__FUNCTION__,pickStop);
      //fprintf(stderr,"after collect: theREf ref=%d sglp.l=%d\n",theRef,sglp[0].l);
      //      exit(0);
      if(0){
	fprintf(stderr,"\n");
	for(int i=0;i<nFiles;i++){
	  fprintf(stderr,"[%s] sgl[%d].l=%d (%d,%d)\n",__FUNCTION__,i,sglp[i].l,sglp[i].first[0],sglp[i].last[sglp[i].l-1]);
	}
	for(int i=0;i<nFiles;i++){
	  fprintf(stderr,"[%s] i=%d nps.l=%d",__FUNCTION__,i,nps[i].l);
	  if(nps[i].l!=0)
	    fprintf(stderr,"(%d,%d)\n",nps[i].first,nps[i].last);
	  else
	    fprintf(stderr,"\n");
	}
      }
      //prepare uppiling for all data, if end of chromosome or all files are EOF
      //fprintf(stderr,"doFlush=%d\tnotDone=%d\n",doFlush,notDone);
      if(doFlush||notDone==0){
	//	fprintf(stderr,"[%s]probs with stuff maybe last chunk Will flush all remaining reads\n",__FUNCTION__);
	getMaxMax2T(sglp,nFiles,nps);
      }else{
	//getSglStop returns the sum of reads to parse.
	int hasData = getSglStop5(sglp,nFiles,pickStop);
	//	fprintf(stderr,"hasDAta=%d\n",hasData);
	if(hasData==0&&notDone!=0){
	  fprintf(stderr,"Triggering extra filereading (before uppiling) reads:notDone=%d SIG_COND=%d\n",notDone,SIG_COND);
	  continue;
	}
      }
      
      //sexit(0);
      //make uppile nodes
      nodePoolT *dn = new nodePoolT[nFiles];
      for(int i=0;i<nFiles;i++){
	dn[i] = mkNodePool_memsafe2_mpileup2T(&sglp[i],&nps[i]);
	//	fprintf(stderr,"AFTER mknodepool[%d] first=%d last=%d\n",i,dn[i].first,dn[i].last);
      }
      if(0){
	for(int i=0;i<nFiles;i++)
	  fprintf(stderr,"l=%d first=%d last=%d\n",dn[i].l,dn[i].first,dn[i].last);

      }

      //fprintf(stderr,"[%s] after memsafe2\n",__FUNCTION__);
      
      //simple sanity check for validating that we have indeed something to print.
      int tmpSum = 0;
      for(int i=0;i<nFiles;i++){
	//	fprintf(stderr,"i=%d\n",i);
	tmpSum += dn[i].l;
	//	printIter(rd[i].it,stderr);
	if(0&& dn[i].l>10000){
	  fprintf(stderr,"seqgfaul %d=%d\n",i,dn[i].l);
	  fprintf(stderr,"fname=%s\n",rd[i].fname);
	  printIter(rd[i].it,stderr);
	  exit(0);
	}
      }
      if(tmpSum==0){

	if(rd[0].regions.size()!=0)
	  notDone=1;
	else{
	fprintf(stderr,"No data for chromoId=%d chromoname=%s\n",theRef,rd[0].hd->name[theRef]);
	fprintf(stderr,"This could either indicate that there really is no data for this chromosome\n");
	fprintf(stderr,"Or it could be problem with this program regSize=%lu notDone=%d\n",rd[0].regions.size(),notDone);
	}
	break;

      }

      //merge the perFile upnodes.FIXME for large regions (with gaps) this will allocate to much...
      //spawns threads
      if(doThread)
	splitfucker_wrapperT(dn,nFiles,regStart,regStop,rd[0].hd->name[theRef],NULL);
      else
	splitfucker_wrapperT_noThread(dn,nFiles,regStart,regStop,rd[0].hd->name[theRef]);
      
  
      //if we flush, its due to end of chr/region or end of files
      if(doFlush)
	break;

    }
    
  }
  //  fprintf(stderr,"after mainloop in function:%s\n",__FUNCTION__);
  callBack_bambi(NULL);//cleanup signal
  fflush(stderr);
  pthread_mutex_lock(&mUpPile_mutex);//just to make sure, its okey to clean up
  //  pthread_mutex_unlock(&mUpPile_mutex);//we call splitfucker later on to clean up the rest
  //  (void) pthread_join(thread, NULL);
  //clean up
  for(int i=0;1&&i<nFiles;i++){
    dalloc(rd[i].it.dasIndex);
    dalloc_nodePoolT(nps[i]);
    delete [] nps[i].nds;
    dalloc(sglp[i]);
  }
  //  splitfucker_wrapperT(NULL,0,0,0,NULL,NULL);
  delete [] nps;
  delete [] sglp;
  return 0;
}


bufReader *mUpPile_rd =NULL;

int motherUppile(int nThreads,bufReader *rd,int type,int nLines,int nFiles,chunk*(*fun)(chunky *),int show) {
  mUpPile_rd = rd;
  //
  if(show==1)
    motherUppile_new3_mpileup_doneNodesReinit(type,nThreads,rd,nLines,nFiles,fun);
  //    motherUppile_new3_mpileup(type,nThreads,rd,nLines,nFiles,fun);
  else{
    //    motherUppile_new3_mpileup_doneNodesReinit_respawn(type,nThreads,rd,nLines,nFiles,fun);
    
    //gen_likesObj->init(rd,nFiles,GLtype);
    faster(type,nThreads,rd,nLines,nFiles,fun); //this function call willl run as long as there is data
    //    gen_likesObj->call_likes(NULL,NULL);//This is essentially a kill signal
    //    callBack_bambi(NULL);//cleanup signal this is now in faster()
  }
  //all_reading is done

  //  fprintf(stderr,"[%s]actuallyReads=%d\n",__FUNCTION__,actuallyReads);
  //  delete gen_likesObj;
  return 0;
}

int motherView(bufReader *rd,int nFiles,chunk*(*fun)(chunky *)) {
  aRead b;
    b.vDat=new uint8_t[RLEN];
    kstring_t str;  str.s=NULL; str.l=str.m=0;

    if(rd[0].regions.size()==0) {//print all
      int block_size;
      while(SIG_COND && bgzf_read(rd[0].fp,&block_size,sizeof(int))){
	getAlign(rd[0].fp,block_size,b);
	printReadBuffered(b,rd[0].hd,str);
	fprintf(stdout,"%s",str.s);

      }
      //      fprintf(stderr,"adsfadf\n");          

    }else {
      //      fprintf(stderr,"The alternative\n");
      for(int i=0;i<(int)rd[0].regions.size();i++){
	int tmpRef = rd[0].regions[i].refID;
	int tmpStart = rd[0].regions[i].start;
	int tmpStop = rd[0].regions[i].stop;
      
	getOffsets(rd[0].fname,rd[0].hd,rd[0].it,tmpRef,tmpStart,tmpStop);
	int ret =0;
	while(SIG_COND){
	  ret = bam_iter_read(rd[0].fp, &rd[0].it, b);
	  if(ret<0)
	    break;
	  printReadBuffered(b,rd[0].hd,str);
	  fprintf(stdout,"%s",str.s);
	}
	free(rd[0].it.off);//the offsets
      }
      free(str.s);
      delete [] b.vDat;
    }

  return 0;
}
